""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class WrapRegion(enum.IntEnum):
    """Indicates source type to extract wrapper region.
    """
    MATERIALPOINT = 0
    """Option to use live material points to extract wrapper region."""
    EXTERNAL = 1
    """Option to extract exterior wrapper region."""
    LARGESTINTERNAL = 2
    """Option to extract largest internal wrapper region."""

class WrapParams(CoreObject):
    """WrapParams defines parameters for wrapping.
    """
    _default_params = {}

    def __initialize(
            self,
            sizing_method: SizeFieldType,
            base_size: float,
            size_control_ids: Iterable[int],
            size_field_ids: Iterable[int],
            wrap_region: WrapRegion,
            number_of_threads: int,
            imprint_relative_range: float,
            imprint_iterations: int):
        self._sizing_method = SizeFieldType(sizing_method)
        self._base_size = base_size
        self._size_control_ids = size_control_ids if isinstance(size_control_ids, np.ndarray) else np.array(size_control_ids, dtype=np.int32) if size_control_ids is not None else None
        self._size_field_ids = size_field_ids if isinstance(size_field_ids, np.ndarray) else np.array(size_field_ids, dtype=np.int32) if size_field_ids is not None else None
        self._wrap_region = WrapRegion(wrap_region)
        self._number_of_threads = number_of_threads
        self._imprint_relative_range = imprint_relative_range
        self._imprint_iterations = imprint_iterations

    def __init__(
            self,
            model: CommunicationManager=None,
            sizing_method: SizeFieldType = None,
            base_size: float = None,
            size_control_ids: Iterable[int] = None,
            size_field_ids: Iterable[int] = None,
            wrap_region: WrapRegion = None,
            number_of_threads: int = None,
            imprint_relative_range: float = None,
            imprint_iterations: int = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the WrapParams.

        Parameters
        ----------
        model: Model
            Model to create a WrapParams object with default parameters.
        sizing_method: SizeFieldType, optional
            Used to define sizing method for wrapping.
        base_size: float, optional
            Base size to define octree.
        size_control_ids: Iterable[int], optional
            Used to construct geodesic sizes for octree refinement.
        size_field_ids: Iterable[int], optional
            Used to define size field based octree refinement.
        wrap_region: WrapRegion, optional
            Indicates source type to extract wrapper region.
        number_of_threads: int, optional
            Number of threads for multithreading.
        imprint_relative_range: float, optional
            Used to define relative range in imprinting in wrapping.
        imprint_iterations: int, optional
            Used to define number of imprint iterations in wrapping.
        json_data: dict, optional
            JSON dictionary to create a WrapParams object with provided parameters.

        Examples
        --------
        >>> wrap_params = prime.WrapParams(model = model)
        """
        if json_data:
            self.__initialize(
                SizeFieldType(json_data["sizingMethod"] if "sizingMethod" in json_data else None),
                json_data["baseSize"] if "baseSize" in json_data else None,
                json_data["sizeControlIDs"] if "sizeControlIDs" in json_data else None,
                json_data["sizeFieldIDs"] if "sizeFieldIDs" in json_data else None,
                WrapRegion(json_data["wrapRegion"] if "wrapRegion" in json_data else None),
                json_data["numberOfThreads"] if "numberOfThreads" in json_data else None,
                json_data["imprintRelativeRange"] if "imprintRelativeRange" in json_data else None,
                json_data["imprintIterations"] if "imprintIterations" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [sizing_method, base_size, size_control_ids, size_field_ids, wrap_region, number_of_threads, imprint_relative_range, imprint_iterations])
            if all_field_specified:
                self.__initialize(
                    sizing_method,
                    base_size,
                    size_control_ids,
                    size_field_ids,
                    wrap_region,
                    number_of_threads,
                    imprint_relative_range,
                    imprint_iterations)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "WrapParams")
                    json_data = param_json["WrapParams"] if "WrapParams" in param_json else {}
                    self.__initialize(
                        sizing_method if sizing_method is not None else ( WrapParams._default_params["sizing_method"] if "sizing_method" in WrapParams._default_params else SizeFieldType(json_data["sizingMethod"] if "sizingMethod" in json_data else None)),
                        base_size if base_size is not None else ( WrapParams._default_params["base_size"] if "base_size" in WrapParams._default_params else (json_data["baseSize"] if "baseSize" in json_data else None)),
                        size_control_ids if size_control_ids is not None else ( WrapParams._default_params["size_control_ids"] if "size_control_ids" in WrapParams._default_params else (json_data["sizeControlIDs"] if "sizeControlIDs" in json_data else None)),
                        size_field_ids if size_field_ids is not None else ( WrapParams._default_params["size_field_ids"] if "size_field_ids" in WrapParams._default_params else (json_data["sizeFieldIDs"] if "sizeFieldIDs" in json_data else None)),
                        wrap_region if wrap_region is not None else ( WrapParams._default_params["wrap_region"] if "wrap_region" in WrapParams._default_params else WrapRegion(json_data["wrapRegion"] if "wrapRegion" in json_data else None)),
                        number_of_threads if number_of_threads is not None else ( WrapParams._default_params["number_of_threads"] if "number_of_threads" in WrapParams._default_params else (json_data["numberOfThreads"] if "numberOfThreads" in json_data else None)),
                        imprint_relative_range if imprint_relative_range is not None else ( WrapParams._default_params["imprint_relative_range"] if "imprint_relative_range" in WrapParams._default_params else (json_data["imprintRelativeRange"] if "imprintRelativeRange" in json_data else None)),
                        imprint_iterations if imprint_iterations is not None else ( WrapParams._default_params["imprint_iterations"] if "imprint_iterations" in WrapParams._default_params else (json_data["imprintIterations"] if "imprintIterations" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            sizing_method: SizeFieldType = None,
            base_size: float = None,
            size_control_ids: Iterable[int] = None,
            size_field_ids: Iterable[int] = None,
            wrap_region: WrapRegion = None,
            number_of_threads: int = None,
            imprint_relative_range: float = None,
            imprint_iterations: int = None):
        """Set the default values of WrapParams.

        Parameters
        ----------
        sizing_method: SizeFieldType, optional
            Used to define sizing method for wrapping.
        base_size: float, optional
            Base size to define octree.
        size_control_ids: Iterable[int], optional
            Used to construct geodesic sizes for octree refinement.
        size_field_ids: Iterable[int], optional
            Used to define size field based octree refinement.
        wrap_region: WrapRegion, optional
            Indicates source type to extract wrapper region.
        number_of_threads: int, optional
            Number of threads for multithreading.
        imprint_relative_range: float, optional
            Used to define relative range in imprinting in wrapping.
        imprint_iterations: int, optional
            Used to define number of imprint iterations in wrapping.
        """
        args = locals()
        [WrapParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of WrapParams.

        Examples
        --------
        >>> WrapParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in WrapParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._sizing_method is not None:
            json_data["sizingMethod"] = self._sizing_method
        if self._base_size is not None:
            json_data["baseSize"] = self._base_size
        if self._size_control_ids is not None:
            json_data["sizeControlIDs"] = self._size_control_ids
        if self._size_field_ids is not None:
            json_data["sizeFieldIDs"] = self._size_field_ids
        if self._wrap_region is not None:
            json_data["wrapRegion"] = self._wrap_region
        if self._number_of_threads is not None:
            json_data["numberOfThreads"] = self._number_of_threads
        if self._imprint_relative_range is not None:
            json_data["imprintRelativeRange"] = self._imprint_relative_range
        if self._imprint_iterations is not None:
            json_data["imprintIterations"] = self._imprint_iterations
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "sizing_method :  %s\nbase_size :  %s\nsize_control_ids :  %s\nsize_field_ids :  %s\nwrap_region :  %s\nnumber_of_threads :  %s\nimprint_relative_range :  %s\nimprint_iterations :  %s" % (self._sizing_method, self._base_size, self._size_control_ids, self._size_field_ids, self._wrap_region, self._number_of_threads, self._imprint_relative_range, self._imprint_iterations)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def sizing_method(self) -> SizeFieldType:
        """Used to define sizing method for wrapping.
        """
        return self._sizing_method

    @sizing_method.setter
    def sizing_method(self, value: SizeFieldType):
        self._sizing_method = value

    @property
    def base_size(self) -> float:
        """Base size to define octree.
        """
        return self._base_size

    @base_size.setter
    def base_size(self, value: float):
        self._base_size = value

    @property
    def size_control_ids(self) -> Iterable[int]:
        """Used to construct geodesic sizes for octree refinement.
        """
        return self._size_control_ids

    @size_control_ids.setter
    def size_control_ids(self, value: Iterable[int]):
        self._size_control_ids = value

    @property
    def size_field_ids(self) -> Iterable[int]:
        """Used to define size field based octree refinement.
        """
        return self._size_field_ids

    @size_field_ids.setter
    def size_field_ids(self, value: Iterable[int]):
        self._size_field_ids = value

    @property
    def wrap_region(self) -> WrapRegion:
        """Indicates source type to extract wrapper region.
        """
        return self._wrap_region

    @wrap_region.setter
    def wrap_region(self, value: WrapRegion):
        self._wrap_region = value

    @property
    def number_of_threads(self) -> int:
        """Number of threads for multithreading.
        """
        return self._number_of_threads

    @number_of_threads.setter
    def number_of_threads(self, value: int):
        self._number_of_threads = value

    @property
    def imprint_relative_range(self) -> float:
        """Used to define relative range in imprinting in wrapping.
        """
        return self._imprint_relative_range

    @imprint_relative_range.setter
    def imprint_relative_range(self, value: float):
        self._imprint_relative_range = value

    @property
    def imprint_iterations(self) -> int:
        """Used to define number of imprint iterations in wrapping.
        """
        return self._imprint_iterations

    @imprint_iterations.setter
    def imprint_iterations(self, value: int):
        self._imprint_iterations = value

class WrapResult(CoreObject):
    """Result structure associated to Wrap operation.
    """
    _default_params = {}

    def __initialize(
            self,
            warning_codes: List[WarningCode],
            error_code: ErrorCode,
            id: int,
            name: str):
        self._warning_codes = warning_codes
        self._error_code = ErrorCode(error_code)
        self._id = id
        self._name = name

    def __init__(
            self,
            model: CommunicationManager=None,
            warning_codes: List[WarningCode] = None,
            error_code: ErrorCode = None,
            id: int = None,
            name: str = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the WrapResult.

        Parameters
        ----------
        model: Model
            Model to create a WrapResult object with default parameters.
        warning_codes: List[WarningCode], optional
            Warning codes associated with the wrap operation.
        error_code: ErrorCode, optional
            Error code associated with a wrap operation.
        id: int, optional
            Id of the wrapper part created.
        name: str, optional
            Name of wrapper part created.
        json_data: dict, optional
            JSON dictionary to create a WrapResult object with provided parameters.

        Examples
        --------
        >>> wrap_result = prime.WrapResult(model = model)
        """
        if json_data:
            self.__initialize(
                [WarningCode(data) for data in json_data["warningCodes"]] if "warningCodes" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                json_data["id"] if "id" in json_data else None,
                json_data["name"] if "name" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [warning_codes, error_code, id, name])
            if all_field_specified:
                self.__initialize(
                    warning_codes,
                    error_code,
                    id,
                    name)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "WrapResult")
                    json_data = param_json["WrapResult"] if "WrapResult" in param_json else {}
                    self.__initialize(
                        warning_codes if warning_codes is not None else ( WrapResult._default_params["warning_codes"] if "warning_codes" in WrapResult._default_params else [WarningCode(data) for data in (json_data["warningCodes"] if "warningCodes" in json_data else None)]),
                        error_code if error_code is not None else ( WrapResult._default_params["error_code"] if "error_code" in WrapResult._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        id if id is not None else ( WrapResult._default_params["id"] if "id" in WrapResult._default_params else (json_data["id"] if "id" in json_data else None)),
                        name if name is not None else ( WrapResult._default_params["name"] if "name" in WrapResult._default_params else (json_data["name"] if "name" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            warning_codes: List[WarningCode] = None,
            error_code: ErrorCode = None,
            id: int = None,
            name: str = None):
        """Set the default values of WrapResult.

        Parameters
        ----------
        warning_codes: List[WarningCode], optional
            Warning codes associated with the wrap operation.
        error_code: ErrorCode, optional
            Error code associated with a wrap operation.
        id: int, optional
            Id of the wrapper part created.
        name: str, optional
            Name of wrapper part created.
        """
        args = locals()
        [WrapResult._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of WrapResult.

        Examples
        --------
        >>> WrapResult.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in WrapResult._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._warning_codes is not None:
            json_data["warningCodes"] = [data for data in self._warning_codes]
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._id is not None:
            json_data["id"] = self._id
        if self._name is not None:
            json_data["name"] = self._name
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "warning_codes :  %s\nerror_code :  %s\nid :  %s\nname :  %s" % ('[' + ''.join('\n' + str(data) for data in self._warning_codes) + ']', self._error_code, self._id, self._name)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def warning_codes(self) -> List[WarningCode]:
        """Warning codes associated with the wrap operation.
        """
        return self._warning_codes

    @warning_codes.setter
    def warning_codes(self, value: List[WarningCode]):
        self._warning_codes = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with a wrap operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def id(self) -> int:
        """Id of the wrapper part created.
        """
        return self._id

    @id.setter
    def id(self, value: int):
        self._id = value

    @property
    def name(self) -> str:
        """Name of wrapper part created.
        """
        return self._name

    @name.setter
    def name(self, value: str):
        self._name = value

class WrapperImproveQualityParams(CoreObject):
    """WrapperImproveQualityParams defines parameters to improve wrapper part.
    """
    _default_params = {}

    def __initialize(
            self,
            target_skewness: float,
            island_count: int,
            island_tol: float,
            overlap_count: int,
            overlap_tol: float,
            resolve_spikes: bool,
            resolve_intersections: bool,
            inflate_dihedral_face_nodes: bool,
            resolve_invalid_node_normals: bool,
            aggressively: bool,
            sharp_angle: float,
            geom_zonelets: Iterable[int],
            number_of_threads: int):
        self._target_skewness = target_skewness
        self._island_count = island_count
        self._island_tol = island_tol
        self._overlap_count = overlap_count
        self._overlap_tol = overlap_tol
        self._resolve_spikes = resolve_spikes
        self._resolve_intersections = resolve_intersections
        self._inflate_dihedral_face_nodes = inflate_dihedral_face_nodes
        self._resolve_invalid_node_normals = resolve_invalid_node_normals
        self._aggressively = aggressively
        self._sharp_angle = sharp_angle
        self._geom_zonelets = geom_zonelets if isinstance(geom_zonelets, np.ndarray) else np.array(geom_zonelets, dtype=np.int32) if geom_zonelets is not None else None
        self._number_of_threads = number_of_threads

    def __init__(
            self,
            model: CommunicationManager=None,
            target_skewness: float = None,
            island_count: int = None,
            island_tol: float = None,
            overlap_count: int = None,
            overlap_tol: float = None,
            resolve_spikes: bool = None,
            resolve_intersections: bool = None,
            inflate_dihedral_face_nodes: bool = None,
            resolve_invalid_node_normals: bool = None,
            aggressively: bool = None,
            sharp_angle: float = None,
            geom_zonelets: Iterable[int] = None,
            number_of_threads: int = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the WrapperImproveQualityParams.

        Parameters
        ----------
        model: Model
            Model to create a WrapperImproveQualityParams object with default parameters.
        target_skewness: float, optional
            Target skewness.
        island_count: int, optional
            Face count of smallest island.
        island_tol: float, optional
            Relative face count of smallest island.
        overlap_count: int, optional
            Face count of non-manifold overlap.
        overlap_tol: float, optional
            Relative face count of non-manifold overlap.
        resolve_spikes: bool, optional
            Control to perform removing spikes or not.
        resolve_intersections: bool, optional
            Control to resolve face intersections or not.
        inflate_dihedral_face_nodes: bool, optional
            Control to resolve face dihedral angle by inflating opposite nodes or not.
        resolve_invalid_node_normals: bool, optional
            Control to resolve invalid node normals by inflating opposite nodes or not.
        aggressively: bool, optional
            Control to improve surfaces aggressively or not.
        sharp_angle: float, optional
            Maximum off feature sharp node angle.
        geom_zonelets: Iterable[int], optional
            Associated underlying geometry zonelet ids.
        number_of_threads: int, optional
            Number of threads for multithreading.
        json_data: dict, optional
            JSON dictionary to create a WrapperImproveQualityParams object with provided parameters.

        Examples
        --------
        >>> wrapper_improve_quality_params = prime.WrapperImproveQualityParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["targetSkewness"] if "targetSkewness" in json_data else None,
                json_data["islandCount"] if "islandCount" in json_data else None,
                json_data["islandTol"] if "islandTol" in json_data else None,
                json_data["overlapCount"] if "overlapCount" in json_data else None,
                json_data["overlapTol"] if "overlapTol" in json_data else None,
                json_data["resolveSpikes"] if "resolveSpikes" in json_data else None,
                json_data["resolveIntersections"] if "resolveIntersections" in json_data else None,
                json_data["inflateDihedralFaceNodes"] if "inflateDihedralFaceNodes" in json_data else None,
                json_data["resolveInvalidNodeNormals"] if "resolveInvalidNodeNormals" in json_data else None,
                json_data["aggressively"] if "aggressively" in json_data else None,
                json_data["sharpAngle"] if "sharpAngle" in json_data else None,
                json_data["geomZonelets"] if "geomZonelets" in json_data else None,
                json_data["numberOfThreads"] if "numberOfThreads" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [target_skewness, island_count, island_tol, overlap_count, overlap_tol, resolve_spikes, resolve_intersections, inflate_dihedral_face_nodes, resolve_invalid_node_normals, aggressively, sharp_angle, geom_zonelets, number_of_threads])
            if all_field_specified:
                self.__initialize(
                    target_skewness,
                    island_count,
                    island_tol,
                    overlap_count,
                    overlap_tol,
                    resolve_spikes,
                    resolve_intersections,
                    inflate_dihedral_face_nodes,
                    resolve_invalid_node_normals,
                    aggressively,
                    sharp_angle,
                    geom_zonelets,
                    number_of_threads)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "WrapperImproveQualityParams")
                    json_data = param_json["WrapperImproveQualityParams"] if "WrapperImproveQualityParams" in param_json else {}
                    self.__initialize(
                        target_skewness if target_skewness is not None else ( WrapperImproveQualityParams._default_params["target_skewness"] if "target_skewness" in WrapperImproveQualityParams._default_params else (json_data["targetSkewness"] if "targetSkewness" in json_data else None)),
                        island_count if island_count is not None else ( WrapperImproveQualityParams._default_params["island_count"] if "island_count" in WrapperImproveQualityParams._default_params else (json_data["islandCount"] if "islandCount" in json_data else None)),
                        island_tol if island_tol is not None else ( WrapperImproveQualityParams._default_params["island_tol"] if "island_tol" in WrapperImproveQualityParams._default_params else (json_data["islandTol"] if "islandTol" in json_data else None)),
                        overlap_count if overlap_count is not None else ( WrapperImproveQualityParams._default_params["overlap_count"] if "overlap_count" in WrapperImproveQualityParams._default_params else (json_data["overlapCount"] if "overlapCount" in json_data else None)),
                        overlap_tol if overlap_tol is not None else ( WrapperImproveQualityParams._default_params["overlap_tol"] if "overlap_tol" in WrapperImproveQualityParams._default_params else (json_data["overlapTol"] if "overlapTol" in json_data else None)),
                        resolve_spikes if resolve_spikes is not None else ( WrapperImproveQualityParams._default_params["resolve_spikes"] if "resolve_spikes" in WrapperImproveQualityParams._default_params else (json_data["resolveSpikes"] if "resolveSpikes" in json_data else None)),
                        resolve_intersections if resolve_intersections is not None else ( WrapperImproveQualityParams._default_params["resolve_intersections"] if "resolve_intersections" in WrapperImproveQualityParams._default_params else (json_data["resolveIntersections"] if "resolveIntersections" in json_data else None)),
                        inflate_dihedral_face_nodes if inflate_dihedral_face_nodes is not None else ( WrapperImproveQualityParams._default_params["inflate_dihedral_face_nodes"] if "inflate_dihedral_face_nodes" in WrapperImproveQualityParams._default_params else (json_data["inflateDihedralFaceNodes"] if "inflateDihedralFaceNodes" in json_data else None)),
                        resolve_invalid_node_normals if resolve_invalid_node_normals is not None else ( WrapperImproveQualityParams._default_params["resolve_invalid_node_normals"] if "resolve_invalid_node_normals" in WrapperImproveQualityParams._default_params else (json_data["resolveInvalidNodeNormals"] if "resolveInvalidNodeNormals" in json_data else None)),
                        aggressively if aggressively is not None else ( WrapperImproveQualityParams._default_params["aggressively"] if "aggressively" in WrapperImproveQualityParams._default_params else (json_data["aggressively"] if "aggressively" in json_data else None)),
                        sharp_angle if sharp_angle is not None else ( WrapperImproveQualityParams._default_params["sharp_angle"] if "sharp_angle" in WrapperImproveQualityParams._default_params else (json_data["sharpAngle"] if "sharpAngle" in json_data else None)),
                        geom_zonelets if geom_zonelets is not None else ( WrapperImproveQualityParams._default_params["geom_zonelets"] if "geom_zonelets" in WrapperImproveQualityParams._default_params else (json_data["geomZonelets"] if "geomZonelets" in json_data else None)),
                        number_of_threads if number_of_threads is not None else ( WrapperImproveQualityParams._default_params["number_of_threads"] if "number_of_threads" in WrapperImproveQualityParams._default_params else (json_data["numberOfThreads"] if "numberOfThreads" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            target_skewness: float = None,
            island_count: int = None,
            island_tol: float = None,
            overlap_count: int = None,
            overlap_tol: float = None,
            resolve_spikes: bool = None,
            resolve_intersections: bool = None,
            inflate_dihedral_face_nodes: bool = None,
            resolve_invalid_node_normals: bool = None,
            aggressively: bool = None,
            sharp_angle: float = None,
            geom_zonelets: Iterable[int] = None,
            number_of_threads: int = None):
        """Set the default values of WrapperImproveQualityParams.

        Parameters
        ----------
        target_skewness: float, optional
            Target skewness.
        island_count: int, optional
            Face count of smallest island.
        island_tol: float, optional
            Relative face count of smallest island.
        overlap_count: int, optional
            Face count of non-manifold overlap.
        overlap_tol: float, optional
            Relative face count of non-manifold overlap.
        resolve_spikes: bool, optional
            Control to perform removing spikes or not.
        resolve_intersections: bool, optional
            Control to resolve face intersections or not.
        inflate_dihedral_face_nodes: bool, optional
            Control to resolve face dihedral angle by inflating opposite nodes or not.
        resolve_invalid_node_normals: bool, optional
            Control to resolve invalid node normals by inflating opposite nodes or not.
        aggressively: bool, optional
            Control to improve surfaces aggressively or not.
        sharp_angle: float, optional
            Maximum off feature sharp node angle.
        geom_zonelets: Iterable[int], optional
            Associated underlying geometry zonelet ids.
        number_of_threads: int, optional
            Number of threads for multithreading.
        """
        args = locals()
        [WrapperImproveQualityParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of WrapperImproveQualityParams.

        Examples
        --------
        >>> WrapperImproveQualityParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in WrapperImproveQualityParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._target_skewness is not None:
            json_data["targetSkewness"] = self._target_skewness
        if self._island_count is not None:
            json_data["islandCount"] = self._island_count
        if self._island_tol is not None:
            json_data["islandTol"] = self._island_tol
        if self._overlap_count is not None:
            json_data["overlapCount"] = self._overlap_count
        if self._overlap_tol is not None:
            json_data["overlapTol"] = self._overlap_tol
        if self._resolve_spikes is not None:
            json_data["resolveSpikes"] = self._resolve_spikes
        if self._resolve_intersections is not None:
            json_data["resolveIntersections"] = self._resolve_intersections
        if self._inflate_dihedral_face_nodes is not None:
            json_data["inflateDihedralFaceNodes"] = self._inflate_dihedral_face_nodes
        if self._resolve_invalid_node_normals is not None:
            json_data["resolveInvalidNodeNormals"] = self._resolve_invalid_node_normals
        if self._aggressively is not None:
            json_data["aggressively"] = self._aggressively
        if self._sharp_angle is not None:
            json_data["sharpAngle"] = self._sharp_angle
        if self._geom_zonelets is not None:
            json_data["geomZonelets"] = self._geom_zonelets
        if self._number_of_threads is not None:
            json_data["numberOfThreads"] = self._number_of_threads
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "target_skewness :  %s\nisland_count :  %s\nisland_tol :  %s\noverlap_count :  %s\noverlap_tol :  %s\nresolve_spikes :  %s\nresolve_intersections :  %s\ninflate_dihedral_face_nodes :  %s\nresolve_invalid_node_normals :  %s\naggressively :  %s\nsharp_angle :  %s\ngeom_zonelets :  %s\nnumber_of_threads :  %s" % (self._target_skewness, self._island_count, self._island_tol, self._overlap_count, self._overlap_tol, self._resolve_spikes, self._resolve_intersections, self._inflate_dihedral_face_nodes, self._resolve_invalid_node_normals, self._aggressively, self._sharp_angle, self._geom_zonelets, self._number_of_threads)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def target_skewness(self) -> float:
        """Target skewness.
        """
        return self._target_skewness

    @target_skewness.setter
    def target_skewness(self, value: float):
        self._target_skewness = value

    @property
    def island_count(self) -> int:
        """Face count of smallest island.
        """
        return self._island_count

    @island_count.setter
    def island_count(self, value: int):
        self._island_count = value

    @property
    def island_tol(self) -> float:
        """Relative face count of smallest island.
        """
        return self._island_tol

    @island_tol.setter
    def island_tol(self, value: float):
        self._island_tol = value

    @property
    def overlap_count(self) -> int:
        """Face count of non-manifold overlap.
        """
        return self._overlap_count

    @overlap_count.setter
    def overlap_count(self, value: int):
        self._overlap_count = value

    @property
    def overlap_tol(self) -> float:
        """Relative face count of non-manifold overlap.
        """
        return self._overlap_tol

    @overlap_tol.setter
    def overlap_tol(self, value: float):
        self._overlap_tol = value

    @property
    def resolve_spikes(self) -> bool:
        """Control to perform removing spikes or not.
        """
        return self._resolve_spikes

    @resolve_spikes.setter
    def resolve_spikes(self, value: bool):
        self._resolve_spikes = value

    @property
    def resolve_intersections(self) -> bool:
        """Control to resolve face intersections or not.
        """
        return self._resolve_intersections

    @resolve_intersections.setter
    def resolve_intersections(self, value: bool):
        self._resolve_intersections = value

    @property
    def inflate_dihedral_face_nodes(self) -> bool:
        """Control to resolve face dihedral angle by inflating opposite nodes or not.
        """
        return self._inflate_dihedral_face_nodes

    @inflate_dihedral_face_nodes.setter
    def inflate_dihedral_face_nodes(self, value: bool):
        self._inflate_dihedral_face_nodes = value

    @property
    def resolve_invalid_node_normals(self) -> bool:
        """Control to resolve invalid node normals by inflating opposite nodes or not.
        """
        return self._resolve_invalid_node_normals

    @resolve_invalid_node_normals.setter
    def resolve_invalid_node_normals(self, value: bool):
        self._resolve_invalid_node_normals = value

    @property
    def aggressively(self) -> bool:
        """Control to improve surfaces aggressively or not.
        """
        return self._aggressively

    @aggressively.setter
    def aggressively(self, value: bool):
        self._aggressively = value

    @property
    def sharp_angle(self) -> float:
        """Maximum off feature sharp node angle.
        """
        return self._sharp_angle

    @sharp_angle.setter
    def sharp_angle(self, value: float):
        self._sharp_angle = value

    @property
    def geom_zonelets(self) -> Iterable[int]:
        """Associated underlying geometry zonelet ids.
        """
        return self._geom_zonelets

    @geom_zonelets.setter
    def geom_zonelets(self, value: Iterable[int]):
        self._geom_zonelets = value

    @property
    def number_of_threads(self) -> int:
        """Number of threads for multithreading.
        """
        return self._number_of_threads

    @number_of_threads.setter
    def number_of_threads(self, value: int):
        self._number_of_threads = value

class WrapperImproveResult(CoreObject):
    """Results structure associated to improve quality.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            n_skew_found: int,
            remaining_skew_faces: int,
            n_face_intersections_found: int,
            unresolved_face_intersections: int):
        self._error_code = ErrorCode(error_code)
        self._n_skew_found = n_skew_found
        self._remaining_skew_faces = remaining_skew_faces
        self._n_face_intersections_found = n_face_intersections_found
        self._unresolved_face_intersections = unresolved_face_intersections

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            n_skew_found: int = None,
            remaining_skew_faces: int = None,
            n_face_intersections_found: int = None,
            unresolved_face_intersections: int = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the WrapperImproveResult.

        Parameters
        ----------
        model: Model
            Model to create a WrapperImproveResult object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with a wrapper operation.
        n_skew_found: int, optional
            Number of skewed faces found.
        remaining_skew_faces: int, optional
            Number of remaining skew faces.
        n_face_intersections_found: int, optional
            Number of self intersections found.
        unresolved_face_intersections: int, optional
            Number of remaining self intersections.
        json_data: dict, optional
            JSON dictionary to create a WrapperImproveResult object with provided parameters.

        Examples
        --------
        >>> wrapper_improve_result = prime.WrapperImproveResult(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                json_data["nSkewFound"] if "nSkewFound" in json_data else None,
                json_data["remainingSkewFaces"] if "remainingSkewFaces" in json_data else None,
                json_data["nFaceIntersectionsFound"] if "nFaceIntersectionsFound" in json_data else None,
                json_data["unresolvedFaceIntersections"] if "unresolvedFaceIntersections" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, n_skew_found, remaining_skew_faces, n_face_intersections_found, unresolved_face_intersections])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    n_skew_found,
                    remaining_skew_faces,
                    n_face_intersections_found,
                    unresolved_face_intersections)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "WrapperImproveResult")
                    json_data = param_json["WrapperImproveResult"] if "WrapperImproveResult" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( WrapperImproveResult._default_params["error_code"] if "error_code" in WrapperImproveResult._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        n_skew_found if n_skew_found is not None else ( WrapperImproveResult._default_params["n_skew_found"] if "n_skew_found" in WrapperImproveResult._default_params else (json_data["nSkewFound"] if "nSkewFound" in json_data else None)),
                        remaining_skew_faces if remaining_skew_faces is not None else ( WrapperImproveResult._default_params["remaining_skew_faces"] if "remaining_skew_faces" in WrapperImproveResult._default_params else (json_data["remainingSkewFaces"] if "remainingSkewFaces" in json_data else None)),
                        n_face_intersections_found if n_face_intersections_found is not None else ( WrapperImproveResult._default_params["n_face_intersections_found"] if "n_face_intersections_found" in WrapperImproveResult._default_params else (json_data["nFaceIntersectionsFound"] if "nFaceIntersectionsFound" in json_data else None)),
                        unresolved_face_intersections if unresolved_face_intersections is not None else ( WrapperImproveResult._default_params["unresolved_face_intersections"] if "unresolved_face_intersections" in WrapperImproveResult._default_params else (json_data["unresolvedFaceIntersections"] if "unresolvedFaceIntersections" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            n_skew_found: int = None,
            remaining_skew_faces: int = None,
            n_face_intersections_found: int = None,
            unresolved_face_intersections: int = None):
        """Set the default values of WrapperImproveResult.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with a wrapper operation.
        n_skew_found: int, optional
            Number of skewed faces found.
        remaining_skew_faces: int, optional
            Number of remaining skew faces.
        n_face_intersections_found: int, optional
            Number of self intersections found.
        unresolved_face_intersections: int, optional
            Number of remaining self intersections.
        """
        args = locals()
        [WrapperImproveResult._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of WrapperImproveResult.

        Examples
        --------
        >>> WrapperImproveResult.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in WrapperImproveResult._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._n_skew_found is not None:
            json_data["nSkewFound"] = self._n_skew_found
        if self._remaining_skew_faces is not None:
            json_data["remainingSkewFaces"] = self._remaining_skew_faces
        if self._n_face_intersections_found is not None:
            json_data["nFaceIntersectionsFound"] = self._n_face_intersections_found
        if self._unresolved_face_intersections is not None:
            json_data["unresolvedFaceIntersections"] = self._unresolved_face_intersections
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\nn_skew_found :  %s\nremaining_skew_faces :  %s\nn_face_intersections_found :  %s\nunresolved_face_intersections :  %s" % (self._error_code, self._n_skew_found, self._remaining_skew_faces, self._n_face_intersections_found, self._unresolved_face_intersections)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with a wrapper operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def n_skew_found(self) -> int:
        """Number of skewed faces found.
        """
        return self._n_skew_found

    @n_skew_found.setter
    def n_skew_found(self, value: int):
        self._n_skew_found = value

    @property
    def remaining_skew_faces(self) -> int:
        """Number of remaining skew faces.
        """
        return self._remaining_skew_faces

    @remaining_skew_faces.setter
    def remaining_skew_faces(self, value: int):
        self._remaining_skew_faces = value

    @property
    def n_face_intersections_found(self) -> int:
        """Number of self intersections found.
        """
        return self._n_face_intersections_found

    @n_face_intersections_found.setter
    def n_face_intersections_found(self, value: int):
        self._n_face_intersections_found = value

    @property
    def unresolved_face_intersections(self) -> int:
        """Number of remaining self intersections.
        """
        return self._unresolved_face_intersections

    @unresolved_face_intersections.setter
    def unresolved_face_intersections(self, value: int):
        self._unresolved_face_intersections = value

class WrapperCloseGapsParams(CoreObject):
    """
    WrapperCloseGapsParams to define parameters for close gaps operation.
    """
    _default_params = {}

    def __initialize(
            self,
            target: ScopeDefinition,
            gap_size: float,
            material_point_name: str,
            suggested_part_name: str,
            number_of_threads: int,
            create_new_part: bool,
            resolution_factor: float):
        self._target = target
        self._gap_size = gap_size
        self._material_point_name = material_point_name
        self._suggested_part_name = suggested_part_name
        self._number_of_threads = number_of_threads
        self._create_new_part = create_new_part
        self._resolution_factor = resolution_factor

    def __init__(
            self,
            model: CommunicationManager=None,
            target: ScopeDefinition = None,
            gap_size: float = None,
            material_point_name: str = None,
            suggested_part_name: str = None,
            number_of_threads: int = None,
            create_new_part: bool = None,
            resolution_factor: float = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the WrapperCloseGapsParams.

        Parameters
        ----------
        model: Model
            Model to create a WrapperCloseGapsParams object with default parameters.
        target: ScopeDefinition, optional
            Scope of target face zonelets to patch gaps between scope and target. If empty scope is provided, CloseGaps patch gaps within scope.
        gap_size: float, optional
            Maximum gap size to be closed.
        material_point_name: str, optional
            Material point name near the gaps to be closed.
        suggested_part_name: str, optional
            Suggested part name for created patching surfaces.
        number_of_threads: int, optional
            Number of threads for multithreading.
        create_new_part: bool, optional
            Creates a new gap closure part. If set to false, merge the patches to the adjacent face zonelet with the highest face count in the input.
        resolution_factor: float, optional
            Factor to resolve the smallest gap for the given gap size.
        json_data: dict, optional
            JSON dictionary to create a WrapperCloseGapsParams object with provided parameters.

        Examples
        --------
        >>> wrapper_close_gaps_params = prime.WrapperCloseGapsParams(model = model)
        """
        if json_data:
            self.__initialize(
                ScopeDefinition(model = model, json_data = json_data["target"] if "target" in json_data else None),
                json_data["gapSize"] if "gapSize" in json_data else None,
                json_data["materialPointName"] if "materialPointName" in json_data else None,
                json_data["suggestedPartName"] if "suggestedPartName" in json_data else None,
                json_data["numberOfThreads"] if "numberOfThreads" in json_data else None,
                json_data["createNewPart"] if "createNewPart" in json_data else None,
                json_data["resolutionFactor"] if "resolutionFactor" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [target, gap_size, material_point_name, suggested_part_name, number_of_threads, create_new_part, resolution_factor])
            if all_field_specified:
                self.__initialize(
                    target,
                    gap_size,
                    material_point_name,
                    suggested_part_name,
                    number_of_threads,
                    create_new_part,
                    resolution_factor)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "WrapperCloseGapsParams")
                    json_data = param_json["WrapperCloseGapsParams"] if "WrapperCloseGapsParams" in param_json else {}
                    self.__initialize(
                        target if target is not None else ( WrapperCloseGapsParams._default_params["target"] if "target" in WrapperCloseGapsParams._default_params else ScopeDefinition(model = model, json_data = (json_data["target"] if "target" in json_data else None))),
                        gap_size if gap_size is not None else ( WrapperCloseGapsParams._default_params["gap_size"] if "gap_size" in WrapperCloseGapsParams._default_params else (json_data["gapSize"] if "gapSize" in json_data else None)),
                        material_point_name if material_point_name is not None else ( WrapperCloseGapsParams._default_params["material_point_name"] if "material_point_name" in WrapperCloseGapsParams._default_params else (json_data["materialPointName"] if "materialPointName" in json_data else None)),
                        suggested_part_name if suggested_part_name is not None else ( WrapperCloseGapsParams._default_params["suggested_part_name"] if "suggested_part_name" in WrapperCloseGapsParams._default_params else (json_data["suggestedPartName"] if "suggestedPartName" in json_data else None)),
                        number_of_threads if number_of_threads is not None else ( WrapperCloseGapsParams._default_params["number_of_threads"] if "number_of_threads" in WrapperCloseGapsParams._default_params else (json_data["numberOfThreads"] if "numberOfThreads" in json_data else None)),
                        create_new_part if create_new_part is not None else ( WrapperCloseGapsParams._default_params["create_new_part"] if "create_new_part" in WrapperCloseGapsParams._default_params else (json_data["createNewPart"] if "createNewPart" in json_data else None)),
                        resolution_factor if resolution_factor is not None else ( WrapperCloseGapsParams._default_params["resolution_factor"] if "resolution_factor" in WrapperCloseGapsParams._default_params else (json_data["resolutionFactor"] if "resolutionFactor" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            target: ScopeDefinition = None,
            gap_size: float = None,
            material_point_name: str = None,
            suggested_part_name: str = None,
            number_of_threads: int = None,
            create_new_part: bool = None,
            resolution_factor: float = None):
        """Set the default values of WrapperCloseGapsParams.

        Parameters
        ----------
        target: ScopeDefinition, optional
            Scope of target face zonelets to patch gaps between scope and target. If empty scope is provided, CloseGaps patch gaps within scope.
        gap_size: float, optional
            Maximum gap size to be closed.
        material_point_name: str, optional
            Material point name near the gaps to be closed.
        suggested_part_name: str, optional
            Suggested part name for created patching surfaces.
        number_of_threads: int, optional
            Number of threads for multithreading.
        create_new_part: bool, optional
            Creates a new gap closure part. If set to false, merge the patches to the adjacent face zonelet with the highest face count in the input.
        resolution_factor: float, optional
            Factor to resolve the smallest gap for the given gap size.
        """
        args = locals()
        [WrapperCloseGapsParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of WrapperCloseGapsParams.

        Examples
        --------
        >>> WrapperCloseGapsParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in WrapperCloseGapsParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._target is not None:
            json_data["target"] = self._target._jsonify()
        if self._gap_size is not None:
            json_data["gapSize"] = self._gap_size
        if self._material_point_name is not None:
            json_data["materialPointName"] = self._material_point_name
        if self._suggested_part_name is not None:
            json_data["suggestedPartName"] = self._suggested_part_name
        if self._number_of_threads is not None:
            json_data["numberOfThreads"] = self._number_of_threads
        if self._create_new_part is not None:
            json_data["createNewPart"] = self._create_new_part
        if self._resolution_factor is not None:
            json_data["resolutionFactor"] = self._resolution_factor
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "target :  %s\ngap_size :  %s\nmaterial_point_name :  %s\nsuggested_part_name :  %s\nnumber_of_threads :  %s\ncreate_new_part :  %s\nresolution_factor :  %s" % ('{ ' + str(self._target) + ' }', self._gap_size, self._material_point_name, self._suggested_part_name, self._number_of_threads, self._create_new_part, self._resolution_factor)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def target(self) -> ScopeDefinition:
        """Scope of target face zonelets to patch gaps between scope and target. If empty scope is provided, CloseGaps patch gaps within scope.
        """
        return self._target

    @target.setter
    def target(self, value: ScopeDefinition):
        self._target = value

    @property
    def gap_size(self) -> float:
        """Maximum gap size to be closed.
        """
        return self._gap_size

    @gap_size.setter
    def gap_size(self, value: float):
        self._gap_size = value

    @property
    def material_point_name(self) -> str:
        """Material point name near the gaps to be closed.
        """
        return self._material_point_name

    @material_point_name.setter
    def material_point_name(self, value: str):
        self._material_point_name = value

    @property
    def suggested_part_name(self) -> str:
        """Suggested part name for created patching surfaces.
        """
        return self._suggested_part_name

    @suggested_part_name.setter
    def suggested_part_name(self, value: str):
        self._suggested_part_name = value

    @property
    def number_of_threads(self) -> int:
        """Number of threads for multithreading.
        """
        return self._number_of_threads

    @number_of_threads.setter
    def number_of_threads(self, value: int):
        self._number_of_threads = value

    @property
    def create_new_part(self) -> bool:
        """Creates a new gap closure part. If set to false, merge the patches to the adjacent face zonelet with the highest face count in the input.
        """
        return self._create_new_part

    @create_new_part.setter
    def create_new_part(self, value: bool):
        self._create_new_part = value

    @property
    def resolution_factor(self) -> float:
        """Factor to resolve the smallest gap for the given gap size.
        """
        return self._resolution_factor

    @resolution_factor.setter
    def resolution_factor(self, value: float):
        self._resolution_factor = value

class WrapperCloseGapsResult(CoreObject):
    """Result structure associated with close gaps operation.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            part_id: int):
        self._error_code = ErrorCode(error_code)
        self._part_id = part_id

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            part_id: int = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the WrapperCloseGapsResult.

        Parameters
        ----------
        model: Model
            Model to create a WrapperCloseGapsResult object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with a close gaps operation.
        part_id: int, optional
            Id of part created with gap cover patches.
        json_data: dict, optional
            JSON dictionary to create a WrapperCloseGapsResult object with provided parameters.

        Examples
        --------
        >>> wrapper_close_gaps_result = prime.WrapperCloseGapsResult(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                json_data["partId"] if "partId" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, part_id])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    part_id)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "WrapperCloseGapsResult")
                    json_data = param_json["WrapperCloseGapsResult"] if "WrapperCloseGapsResult" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( WrapperCloseGapsResult._default_params["error_code"] if "error_code" in WrapperCloseGapsResult._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        part_id if part_id is not None else ( WrapperCloseGapsResult._default_params["part_id"] if "part_id" in WrapperCloseGapsResult._default_params else (json_data["partId"] if "partId" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            part_id: int = None):
        """Set the default values of WrapperCloseGapsResult.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with a close gaps operation.
        part_id: int, optional
            Id of part created with gap cover patches.
        """
        args = locals()
        [WrapperCloseGapsResult._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of WrapperCloseGapsResult.

        Examples
        --------
        >>> WrapperCloseGapsResult.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in WrapperCloseGapsResult._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._part_id is not None:
            json_data["partId"] = self._part_id
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\npart_id :  %s" % (self._error_code, self._part_id)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with a close gaps operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def part_id(self) -> int:
        """Id of part created with gap cover patches.
        """
        return self._part_id

    @part_id.setter
    def part_id(self, value: int):
        self._part_id = value



"""Module for communications with the gRPC server."""
__all__ = ['GRPCCommunicator']
from typing import Optional

import grpc
from ansys.api.meshing.prime.v1 import prime_pb2, prime_pb2_grpc

import ansys.meshing.prime.internals.config as config
import ansys.meshing.prime.internals.defaults as defaults
import ansys.meshing.prime.internals.grpc_utils as grpc_utils
import ansys.meshing.prime.internals.json_utils as json
from ansys.meshing.prime.core.model import Model
from ansys.meshing.prime.internals.communicator import Communicator
from ansys.meshing.prime.internals.error_handling import (
    communicator_error_handler,
    error_code_handler,
)

# Keep some buffer for gRPC metadata that it may want to send
BUFFER_MESSAGE_LENGTH = defaults.max_message_length() - 100


def make_chunks(data, chunk_size):
    n = max(1, chunk_size)
    return (data[i : i + n] for i in range(0, len(data), n))


def request_iterator(model_id, data, message_cls, model_cls, content_cls, completion_cls):
    yield message_cls(model=model_cls(object_id=model_id))
    for chunk in make_chunks(data, BUFFER_MESSAGE_LENGTH):
        yield message_cls(content=content_cls(data=chunk))
    yield message_cls(completion_token=completion_cls())


def get_response_messages(response_generator):
    for response in response_generator:
        if response.HasField('completion_token'):
            break

        assert response.HasField('content')
        yield response.content


def get_response(response_iterator, separator):
    return separator.join(response.data for response in get_response_messages(response_iterator))


class GRPCCommunicator(Communicator):
    """Manages communication with the gRPC server.

    Parameters
    ----------
    ip : Optional[str], optional
        IP address where the server is located. The default is ``None``.
    port : Optional[int], optional
        Port where the server is deployed. The default is ``None``.
    timeout : float, optional
        Maximum time to wait for connection. The default is ``10.0``.
    credentials : Any, optional
        Credentials for connecting to the server. The default is ``None``.

    Raises
    ------
    ConnectionError
        Could not connect to server.
    """

    def __init__(
        self,
        ip: Optional[str] = None,
        port: Optional[int] = None,
        timeout: float = 10.0,
        credentials=None,
        **kwargs,
    ):
        """Initialize the server connection."""
        self._channel = kwargs.get('channel', None)
        self._models = []
        if self._channel is None:
            ip_addr = f"{ip}:{port}"
            channel_options = grpc_utils.get_default_channel_args()
            if credentials is None:
                self._channel = grpc.insecure_channel(ip_addr, options=channel_options)
            else:
                self._channel = grpc.secure_channel(ip_addr, credentials, options=channel_options)
        try:
            grpc.channel_ready_future(self._channel).result(timeout=timeout)
        except grpc.FutureTimeoutError as err:
            raise ConnectionError(
                f'Failed to connect to Server in given timeout of {timeout} secs'
            ) from err

        self._stub = prime_pb2_grpc.PrimeStub(self._channel)
        try:
            response = self._stub.Initialize(prime_pb2.InitializeRequest())
            message = json.loads(response.data)
            if 'ServerError' in message:
                raise ConnectionError(message['ServerError'])
            elif 'Results' in message:
                self._models = message['Results']
        except ConnectionError:
            raise
        except:
            pass

    @error_code_handler
    @communicator_error_handler
    def serve(self, model: Model, command: str, *args, **kwargs) -> dict:
        """Serve model and send a command to the server.

        Parameters
        ----------
        model : Model
            Model to serve.
        command : str
            Command to send.

        Returns
        -------
        dict
            Response from the server.

        Raises
        ------
        RuntimeError
            Can not connect to server.
        """
        if self._stub is not None:
            command = {"Command": command}
            if len(args) > 0:
                command.update({"ObjectID": args[0]})
            if kwargs is not None:
                if "args" in kwargs:
                    command.update({"Args": kwargs["args"]})

            if config.is_optimizing_numpy_arrays():
                response = self._stub.ServeJsonBinary(
                    request_iterator(
                        model._object_id,
                        json.dumps(command),
                        prime_pb2.BinaryMessage,
                        prime_pb2.Model,
                        prime_pb2.BinaryJsonContent,
                        prime_pb2.MessageCompletionToken,
                    )
                )
                message = get_response(response, b'')
            else:
                response = self._stub.ServeJson(
                    request_iterator(
                        model._object_id,
                        json.dumps(command),
                        prime_pb2.StringMessage,
                        prime_pb2.Model,
                        prime_pb2.StringJsonContent,
                        prime_pb2.MessageCompletionToken,
                    )
                )
                message = get_response(response, '')
            if defaults.print_communicator_stats():
                import logging

                logging.getLogger("PyPrimeMesh").info(
                    f'Data Transfer: Received {len(message)} bytes'
                )
            return json.loads(message)
        else:
            raise RuntimeError("No connection with server")

    def initialize_params(self, model: Model, param_name: str, *args) -> dict:
        """Initialize parameters on the server side.

        Parameters
        ----------
        model : Model
            Model to initialize parameters on.
        param_name : str
            Parameter to initialize.

        Returns
        -------
        dict
            Response from the server.

        Raises
        ------
        RuntimeError
            Can not connect to server.
        """
        if self._stub is not None:
            command = {"ParamName": param_name}
            with config.numpy_array_optimization_disabled():
                response = self._stub.GetParamDefaultJson(
                    request_iterator(
                        model._object_id,
                        json.dumps(command),
                        prime_pb2.StringMessage,
                        prime_pb2.Model,
                        prime_pb2.StringJsonContent,
                        prime_pb2.MessageCompletionToken,
                    )
                )
                message = get_response(response, '')
                result = json.loads(message)
            return result
        else:
            raise RuntimeError("No connection with server")

    def run_on_server(self, model: Model, recipe: str) -> dict:
        """Run commands on the server.

        Run commands on a model on the server.

        Parameters
        ----------
        model : core.Model
            Model to run commands on.
        recipe : str
            Commands to run.

        Returns
        -------
        dict
            Result from the server side.

        Raises
        ------
        RuntimeError
            Bad response from server.
        RuntimeError
            Can not connect to server.
        """
        if self._stub is not None:
            commands = recipe
            response = self._stub.RunOnServer(
                request_iterator(
                    model._object_id,
                    commands,
                    prime_pb2.StringMessage,
                    prime_pb2.Model,
                    prime_pb2.StringJsonContent,
                    prime_pb2.MessageCompletionToken,
                )
            )
            message = get_response(response, '')
            with config.numpy_array_optimization_disabled():
                result = json.loads(message)
            if "Error" in result:
                raise RuntimeError(result['Error'])
            return result
        else:
            raise RuntimeError("No connection with server")

    def close(self):
        """Close opened channels."""
        self._stub = None
        if self._channel is not None:
            self._channel.close()
            self._channel = None

    def __del__(self):
        """Close communication when deleting the instance."""
        self.close()

    @property
    def models(self):
        """List of models available."""
        return self._models



""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class DeleteFringesAndOverlapsParams(CoreObject):
    """Parameters to delete fringes and overlapping faces.
    """
    _default_params = {}

    def __initialize(
            self,
            fringe_element_count: int,
            overlap_element_count: int,
            delete_fringes: bool,
            delete_overlaps: bool):
        self._fringe_element_count = fringe_element_count
        self._overlap_element_count = overlap_element_count
        self._delete_fringes = delete_fringes
        self._delete_overlaps = delete_overlaps

    def __init__(
            self,
            model: CommunicationManager=None,
            fringe_element_count: int = None,
            overlap_element_count: int = None,
            delete_fringes: bool = None,
            delete_overlaps: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the DeleteFringesAndOverlapsParams.

        Parameters
        ----------
        model: Model
            Model to create a DeleteFringesAndOverlapsParams object with default parameters.
        fringe_element_count: int, optional
            Maximum count of free face elements identified as fringe to be deleted.
        overlap_element_count: int, optional
            Maximum count of overlapping face elements identified as overlap to be deleted.
        delete_fringes: bool, optional
            Option to delete fringes. The default is true.
        delete_overlaps: bool, optional
            Option to delete overlaps. The default is false.
        json_data: dict, optional
            JSON dictionary to create a DeleteFringesAndOverlapsParams object with provided parameters.

        Examples
        --------
        >>> delete_fringes_and_overlaps_params = prime.DeleteFringesAndOverlapsParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["fringeElementCount"] if "fringeElementCount" in json_data else None,
                json_data["overlapElementCount"] if "overlapElementCount" in json_data else None,
                json_data["deleteFringes"] if "deleteFringes" in json_data else None,
                json_data["deleteOverlaps"] if "deleteOverlaps" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [fringe_element_count, overlap_element_count, delete_fringes, delete_overlaps])
            if all_field_specified:
                self.__initialize(
                    fringe_element_count,
                    overlap_element_count,
                    delete_fringes,
                    delete_overlaps)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "DeleteFringesAndOverlapsParams")
                    json_data = param_json["DeleteFringesAndOverlapsParams"] if "DeleteFringesAndOverlapsParams" in param_json else {}
                    self.__initialize(
                        fringe_element_count if fringe_element_count is not None else ( DeleteFringesAndOverlapsParams._default_params["fringe_element_count"] if "fringe_element_count" in DeleteFringesAndOverlapsParams._default_params else (json_data["fringeElementCount"] if "fringeElementCount" in json_data else None)),
                        overlap_element_count if overlap_element_count is not None else ( DeleteFringesAndOverlapsParams._default_params["overlap_element_count"] if "overlap_element_count" in DeleteFringesAndOverlapsParams._default_params else (json_data["overlapElementCount"] if "overlapElementCount" in json_data else None)),
                        delete_fringes if delete_fringes is not None else ( DeleteFringesAndOverlapsParams._default_params["delete_fringes"] if "delete_fringes" in DeleteFringesAndOverlapsParams._default_params else (json_data["deleteFringes"] if "deleteFringes" in json_data else None)),
                        delete_overlaps if delete_overlaps is not None else ( DeleteFringesAndOverlapsParams._default_params["delete_overlaps"] if "delete_overlaps" in DeleteFringesAndOverlapsParams._default_params else (json_data["deleteOverlaps"] if "deleteOverlaps" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            fringe_element_count: int = None,
            overlap_element_count: int = None,
            delete_fringes: bool = None,
            delete_overlaps: bool = None):
        """Set the default values of DeleteFringesAndOverlapsParams.

        Parameters
        ----------
        fringe_element_count: int, optional
            Maximum count of free face elements identified as fringe to be deleted.
        overlap_element_count: int, optional
            Maximum count of overlapping face elements identified as overlap to be deleted.
        delete_fringes: bool, optional
            Option to delete fringes. The default is true.
        delete_overlaps: bool, optional
            Option to delete overlaps. The default is false.
        """
        args = locals()
        [DeleteFringesAndOverlapsParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of DeleteFringesAndOverlapsParams.

        Examples
        --------
        >>> DeleteFringesAndOverlapsParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in DeleteFringesAndOverlapsParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._fringe_element_count is not None:
            json_data["fringeElementCount"] = self._fringe_element_count
        if self._overlap_element_count is not None:
            json_data["overlapElementCount"] = self._overlap_element_count
        if self._delete_fringes is not None:
            json_data["deleteFringes"] = self._delete_fringes
        if self._delete_overlaps is not None:
            json_data["deleteOverlaps"] = self._delete_overlaps
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "fringe_element_count :  %s\noverlap_element_count :  %s\ndelete_fringes :  %s\ndelete_overlaps :  %s" % (self._fringe_element_count, self._overlap_element_count, self._delete_fringes, self._delete_overlaps)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def fringe_element_count(self) -> int:
        """Maximum count of free face elements identified as fringe to be deleted.
        """
        return self._fringe_element_count

    @fringe_element_count.setter
    def fringe_element_count(self, value: int):
        self._fringe_element_count = value

    @property
    def overlap_element_count(self) -> int:
        """Maximum count of overlapping face elements identified as overlap to be deleted.
        """
        return self._overlap_element_count

    @overlap_element_count.setter
    def overlap_element_count(self, value: int):
        self._overlap_element_count = value

    @property
    def delete_fringes(self) -> bool:
        """Option to delete fringes. The default is true.
        """
        return self._delete_fringes

    @delete_fringes.setter
    def delete_fringes(self, value: bool):
        self._delete_fringes = value

    @property
    def delete_overlaps(self) -> bool:
        """Option to delete overlaps. The default is false.
        """
        return self._delete_overlaps

    @delete_overlaps.setter
    def delete_overlaps(self, value: bool):
        self._delete_overlaps = value

class DeleteFringesAndOverlapsResults(CoreObject):
    """Results associated with the delete fringes and overlapping faces operation.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            n_deleted: int):
        self._error_code = ErrorCode(error_code)
        self._n_deleted = n_deleted

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            n_deleted: int = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the DeleteFringesAndOverlapsResults.

        Parameters
        ----------
        model: Model
            Model to create a DeleteFringesAndOverlapsResults object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with the failure of operation.
        n_deleted: int, optional
            Number of face elements deleted.
        json_data: dict, optional
            JSON dictionary to create a DeleteFringesAndOverlapsResults object with provided parameters.

        Examples
        --------
        >>> delete_fringes_and_overlaps_results = prime.DeleteFringesAndOverlapsResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                json_data["nDeleted"] if "nDeleted" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, n_deleted])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    n_deleted)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "DeleteFringesAndOverlapsResults")
                    json_data = param_json["DeleteFringesAndOverlapsResults"] if "DeleteFringesAndOverlapsResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( DeleteFringesAndOverlapsResults._default_params["error_code"] if "error_code" in DeleteFringesAndOverlapsResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        n_deleted if n_deleted is not None else ( DeleteFringesAndOverlapsResults._default_params["n_deleted"] if "n_deleted" in DeleteFringesAndOverlapsResults._default_params else (json_data["nDeleted"] if "nDeleted" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            n_deleted: int = None):
        """Set the default values of DeleteFringesAndOverlapsResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with the failure of operation.
        n_deleted: int, optional
            Number of face elements deleted.
        """
        args = locals()
        [DeleteFringesAndOverlapsResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of DeleteFringesAndOverlapsResults.

        Examples
        --------
        >>> DeleteFringesAndOverlapsResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in DeleteFringesAndOverlapsResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._n_deleted is not None:
            json_data["nDeleted"] = self._n_deleted
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\nn_deleted :  %s" % (self._error_code, self._n_deleted)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the failure of operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def n_deleted(self) -> int:
        """Number of face elements deleted.
        """
        return self._n_deleted

    @n_deleted.setter
    def n_deleted(self, value: int):
        self._n_deleted = value



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class FileIO(CoreObject):
    """Handles reading or writing files from the disk.

    """

    def __init__(self, model: CommunicationManager):
        """ Initialize FileIO """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::FileIO/Construct"
        args = {"ModelID" : model._object_id , "MaxID" : -1 }
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for FileIO. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for FileIO. """
        command_name = "PrimeMesh::FileIO/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def read_pmdat(self, file_name : str, file_read_params : FileReadParams) -> FileReadResults:
        """ Function that reads PRIME's database file.

        PRIME's database files have pmdat extension.

        Parameters
        ----------
        file_name : str
            Path to file on disk.
        file_read_params : FileReadParams
            Parameter to read a file.

        Returns
        -------
        FileReadResults
            Returns FileReadResults.


        Notes
        -----
        This API does not support Unicode paths now.

        Examples
        --------
        >>> import ansys.meshing.prime as prime
        >>> #connect client to server and get model from it
        >>> client = prime.Client(ip="localhost", port=50060)
        >>> model = client.model
        >>> file_io = prime.FileIO(model = model)
        >>> file_read_params = prime.FileReadParams(model = model)
        >>> results = file_io.read_pmdat("/tmp/file.pmdat", file_read_params)

        """
        if not isinstance(file_name, str):
            raise TypeError("Invalid argument type passed for file_name, valid argument type is str.")
        if not isinstance(file_read_params, FileReadParams):
            raise TypeError("Invalid argument type passed for file_read_params, valid argument type is FileReadParams.")
        args = {"file_name" : file_name,
        "file_read_params" : file_read_params._jsonify()}
        command_name = "PrimeMesh::FileIO/ReadPMDAT"
        self._model._print_logs_before_command("read_pmdat", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("read_pmdat", FileReadResults(model = self._model, json_data = result))
        return FileReadResults(model = self._model, json_data = result)

    def write_pmdat(self, file_name : str, file_write_params : FileWriteParams) -> FileWriteResults:
        """ Writes Prime mesh data file. Prime mesh data files have .pmdat extension.


        Parameters
        ----------
        file_name : str
            Path to write file on disk.
        file_write_params : FileWriteParams
            Parameters to write Prime mesh data file.

        Returns
        -------
        FileWriteResults
            Returns the FileWriteResults structure.


        Examples
        --------
        >>> results = file_io.write_pmdat("/tmp/prime_mesh_data.pmdat", prime.FileWriteParams(model = model))

        """
        if not isinstance(file_name, str):
            raise TypeError("Invalid argument type passed for file_name, valid argument type is str.")
        if not isinstance(file_write_params, FileWriteParams):
            raise TypeError("Invalid argument type passed for file_write_params, valid argument type is FileWriteParams.")
        args = {"file_name" : file_name,
        "file_write_params" : file_write_params._jsonify()}
        command_name = "PrimeMesh::FileIO/WritePMDAT"
        self._model._print_logs_before_command("write_pmdat", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("write_pmdat", FileWriteResults(model = self._model, json_data = result))
        return FileWriteResults(model = self._model, json_data = result)

    def get_abaqus_simulation_data(self, partId : int) -> str:
        """ Get Simulation document generated by Abaqus import for a given part.

        This method will return the JSON Simulation Document for a part if the part exists. Otherwise,
        it returns an empty string.

        Parameters
        ----------
        partId : int
            Part Id.

        Returns
        -------
        str
            Returns the string containing a JSON document for simulation data.


        Notes
        -----
        This API is a Beta. API Behavior and implementation may change in future.

        Examples
        --------
        >>> import json
        >>> simdata = json.loads(file_io.get_abaqus_simulation_data(2)

        """
        if not isinstance(partId, int):
            raise TypeError("Invalid argument type passed for partId, valid argument type is int.")
        args = {"partId" : partId}
        command_name = "PrimeMesh::FileIO/GetAbaqusSimulationData"
        self._model._print_beta_api_warning("get_abaqus_simulation_data")
        self._model._print_logs_before_command("get_abaqus_simulation_data", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_abaqus_simulation_data")
        return result

    def import_abaqus_inp(self, file_name : str, params : ImportAbaqusParams) -> ImportAbaqusResults:
        """ Import a Abaqus file.

        Import abaqus files as dead mesh and also store simulation-specific information into Prime in the form of JSON documents.

        Parameters
        ----------
        file_name : str
            Name of file to import.
        params : ImportAbaqusParams
            Parameters to specify options during import.

        Returns
        -------
        ImportAbaqusResults
            Returns the results of the abaqus database import.


        Notes
        -----
        This API is a Beta. API Behavior and implementation may change in future.

        Examples
        --------
        >>> results = file_io.import_abaqus_inp(r"/tmp/file.inp")

        """
        if not isinstance(file_name, str):
            raise TypeError("Invalid argument type passed for file_name, valid argument type is str.")
        if not isinstance(params, ImportAbaqusParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is ImportAbaqusParams.")
        args = {"file_name" : file_name,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::FileIO/ImportAbaqusInp"
        self._model._print_beta_api_warning("import_abaqus_inp")
        self._model._print_logs_before_command("import_abaqus_inp", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("import_abaqus_inp", ImportAbaqusResults(model = self._model, json_data = result))
        return ImportAbaqusResults(model = self._model, json_data = result)

    def import_fluent_meshing_size_field(self, file_name : str) -> SizeFieldFileReadResults:
        """ Import Fluent-Meshing's sizefield file from disk.

        Fluent-Meshing's sizefield files have sf and sf.gz extension.

        Parameters
        ----------
        file_name : str
            Path to file on disk

        Returns
        -------
        SizeFieldFileReadResults
            Return the SizeFieldFileReadResults.


        Notes
        -----
        This API does not support Unicode paths now.

        Examples
        --------
        >>> file_io = prime.FileIO(model = model)
        >>> results = file_io.import_fluent_meshing_size_field("/tmp/my_sizefield.sf")

        """
        if not isinstance(file_name, str):
            raise TypeError("Invalid argument type passed for file_name, valid argument type is str.")
        args = {"file_name" : file_name}
        command_name = "PrimeMesh::FileIO/ImportFluentMeshingSizeField"
        self._model._print_logs_before_command("import_fluent_meshing_size_field", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("import_fluent_meshing_size_field", SizeFieldFileReadResults(model = self._model, json_data = result))
        return SizeFieldFileReadResults(model = self._model, json_data = result)

    def read_size_field(self, file_name : str, params : ReadSizeFieldParams) -> SizeFieldFileReadResults:
        """ Read PRIME's sizefield file from disk.

        PRIME's sizefield files have psf and psf.gz extension.

        Parameters
        ----------
        file_name : str
            Path to file on disk.
        params : ReadSizeFieldParams
            Parameters to read size field file.

        Returns
        -------
        SizeFieldFileReadResults
            Return the SizeFieldFileReadResults.


        Notes
        -----
        This API does not support Unicode paths now.

        Examples
        --------
        >>> file_io = prime.FileIO(model = model)
        >>> params = prime.ReadSizeFieldParams(model = model)
        >>> results = file_io.read_size_field("/tmp/my_prime_sizefield.psf", params)

        """
        if not isinstance(file_name, str):
            raise TypeError("Invalid argument type passed for file_name, valid argument type is str.")
        if not isinstance(params, ReadSizeFieldParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is ReadSizeFieldParams.")
        args = {"file_name" : file_name,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::FileIO/ReadSizeField"
        self._model._print_logs_before_command("read_size_field", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("read_size_field", SizeFieldFileReadResults(model = self._model, json_data = result))
        return SizeFieldFileReadResults(model = self._model, json_data = result)

    def write_size_field(self, file_name : str, params : WriteSizeFieldParams) -> FileWriteResults:
        """ Write PRIME's sizefield (.psf) to file.


        Parameters
        ----------
        file_name : str
            Path to file on disk.
        params : WriteSizeFieldParams
            Parameters to write size field file.

        Returns
        -------
        FileWriteResults
            Return the FileWriteResults.


        Notes
        -----
        This API does not support Unicode paths now.

        Examples
        --------
        >>> file_io = prime.FileIO(model = model)
        >>> params = prime.WriteSizeFieldParams(model = model)
        >>> results = file_io.write_size_field("/tmp/my_prime_sizefield.psf", params)

        """
        if not isinstance(file_name, str):
            raise TypeError("Invalid argument type passed for file_name, valid argument type is str.")
        if not isinstance(params, WriteSizeFieldParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is WriteSizeFieldParams.")
        args = {"file_name" : file_name,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::FileIO/WriteSizeField"
        self._model._print_logs_before_command("write_size_field", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("write_size_field", FileWriteResults(model = self._model, json_data = result))
        return FileWriteResults(model = self._model, json_data = result)

    def import_mapdl_cdb(self, file_name : str, params : ImportMapdlCdbParams) -> ImportMapdlCdbResults:
        """ Function that imports MAPDL CDB file(cdb).


        Parameters
        ----------
        file_name : str
            Path to file on disk.
        params : ImportMapdlCdbParams
            Parameter to import a CDB file.

        Returns
        -------
        ImportMapdlCdbResults
            Returns ImportMapdlCdbResults.


        Notes
        -----
        This API does not support Unicode paths now.

        Examples
        --------
        >>> import ansys.meshing.prime as prime
        >>> #connect client to server and get model from it
        >>> client = prime.Client(ip="localhost", port=50060)
        >>> model = client.model
        >>> file_io = prime.FileIO(model = model)
        >>> params = prime.ImportMapdlCdbParams(model = model)
        >>> results = file_io.import_mapdl_cdb("/tmp/file.cdb", params)

        """
        if not isinstance(file_name, str):
            raise TypeError("Invalid argument type passed for file_name, valid argument type is str.")
        if not isinstance(params, ImportMapdlCdbParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is ImportMapdlCdbParams.")
        args = {"file_name" : file_name,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::FileIO/ImportMapdlCdb"
        self._model._print_logs_before_command("import_mapdl_cdb", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("import_mapdl_cdb", ImportMapdlCdbResults(model = self._model, json_data = result))
        return ImportMapdlCdbResults(model = self._model, json_data = result)

    def export_mapdl_cdb(self, file_name : str, params : ExportMapdlCdbParams) -> ExportMapdlCdbResults:
        """ Function that exports MAPDL CDB file(cdb).


        Parameters
        ----------
        file_name : str
            Path to file on disk.
        params : ExportMapdlCdbParams
            Parameter to export a CDB file.

        Returns
        -------
        ExportMapdlCdbResults
            Returns ExportMapdlCdbResults.


        Notes
        -----
        This API does not support Unicode paths now.

        Examples
        --------
        >>> import ansys.meshing.prime as prime
        >>> #connect client to server and get model from it
        >>> client = prime.Client(ip="localhost", port=50060)
        >>> model = client.model
        >>> file_io = prime.FileIO(model = model)
        >>> params = prime.ExportMapdlCdbParams(model = model)
        >>> results = file_io.export_mapdl_cdb("/tmp/file.cdb", params)

        """
        if not isinstance(file_name, str):
            raise TypeError("Invalid argument type passed for file_name, valid argument type is str.")
        if not isinstance(params, ExportMapdlCdbParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is ExportMapdlCdbParams.")
        args = {"file_name" : file_name,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::FileIO/ExportMapdlCdb"
        self._model._print_logs_before_command("export_mapdl_cdb", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("export_mapdl_cdb", ExportMapdlCdbResults(model = self._model, json_data = result))
        return ExportMapdlCdbResults(model = self._model, json_data = result)

    def import_fluent_meshing_meshes(self, file_names : List[str], import_fluent_meshing_mesh_params : ImportFluentMeshingMeshParams) -> ImportFluentMeshingMeshResults:
        """ Imports fluent meshing meshes of given files on disk.

        Fluent Meshing mesh files have msh and msh.gz extension.

        Parameters
        ----------
        file_names : List[str]
            Full path of files to be imported.
        import_fluent_meshing_mesh_params : ImportFluentMeshingMeshParams
            Parameters to import fluent meshing mesh.

        Returns
        -------
        ImportFluentMeshingMeshResults
            Returns the FileReadResults.


        Notes
        -----
        This API does not support Unicode paths now.

        Examples
        --------
        >>> file_io = prime.FileIO(model = model)
        >>> params = prime.ImportFluentMeshingMeshParams(model = model)
        >>> results = file_io.import_fluent_meshing_meshes(["/tmp/mesh.msh", "/tmp/mesh1.msh"], params)

        """
        if not isinstance(file_names, List):
            raise TypeError("Invalid argument type passed for file_names, valid argument type is List[str].")
        if not isinstance(import_fluent_meshing_mesh_params, ImportFluentMeshingMeshParams):
            raise TypeError("Invalid argument type passed for import_fluent_meshing_mesh_params, valid argument type is ImportFluentMeshingMeshParams.")
        args = {"file_names" : file_names,
        "import_fluent_meshing_mesh_params" : import_fluent_meshing_mesh_params._jsonify()}
        command_name = "PrimeMesh::FileIO/ImportFluentMeshingMeshes"
        self._model._print_logs_before_command("import_fluent_meshing_meshes", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("import_fluent_meshing_meshes", ImportFluentMeshingMeshResults(model = self._model, json_data = result))
        return ImportFluentMeshingMeshResults(model = self._model, json_data = result)

    def import_fluent_case(self, file_name : str, import_fluent_case_params : ImportFluentCaseParams) -> ImportFluentCaseResults:
        """ Imports fluent case file on disk.

        Fluent case files have cas extension.

        Parameters
        ----------
        file_name : str
            Path to file on disk.
        import_fluent_case_params : ImportFluentCaseParams
            Parameters to import fluent case file.

        Returns
        -------
        ImportFluentCaseResults
            Returns the ImportFluentCaseResults.


        Notes
        -----
        This API does not support unicode paths now.

        Examples
        --------
        >>> file_io = prime.FileIO(model = model)
        >>> params = prime.ImportFluentCaseParams(model = model)
        >>> results = file_io.import_fluent_case("/tmp/fluent.cas", params)

        """
        if not isinstance(file_name, str):
            raise TypeError("Invalid argument type passed for file_name, valid argument type is str.")
        if not isinstance(import_fluent_case_params, ImportFluentCaseParams):
            raise TypeError("Invalid argument type passed for import_fluent_case_params, valid argument type is ImportFluentCaseParams.")
        args = {"file_name" : file_name,
        "import_fluent_case_params" : import_fluent_case_params._jsonify()}
        command_name = "PrimeMesh::FileIO/ImportFluentCase"
        self._model._print_logs_before_command("import_fluent_case", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("import_fluent_case", ImportFluentCaseResults(model = self._model, json_data = result))
        return ImportFluentCaseResults(model = self._model, json_data = result)

    def export_fluent_case(self, file_name : str, export_fluent_case_params : ExportFluentCaseParams) -> FileWriteResults:
        """ Exports Fluent case file. Fluent case files have cas extension.


        Parameters
        ----------
        file_name : str
            Path to file on disk.
        export_fluent_case_params : ExportFluentCaseParams
            Parameters to export fluent case file.

        Returns
        -------
        FileWriteResults
            Returns the FileWriteResults structure.


        Examples
        --------
        >>> file_io = FileIO(model = model)
        >>> results = file_io.export_fluent_case("/tmp/fluent.cas", prime.ExportFluentCaseParams(model = model))

        """
        if not isinstance(file_name, str):
            raise TypeError("Invalid argument type passed for file_name, valid argument type is str.")
        if not isinstance(export_fluent_case_params, ExportFluentCaseParams):
            raise TypeError("Invalid argument type passed for export_fluent_case_params, valid argument type is ExportFluentCaseParams.")
        args = {"file_name" : file_name,
        "export_fluent_case_params" : export_fluent_case_params._jsonify()}
        command_name = "PrimeMesh::FileIO/ExportFluentCase"
        self._model._print_logs_before_command("export_fluent_case", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("export_fluent_case", FileWriteResults(model = self._model, json_data = result))
        return FileWriteResults(model = self._model, json_data = result)

    def export_fluent_meshing_mesh(self, file_name : str, export_fluent_mesh_params : ExportFluentMeshingMeshParams) -> FileWriteResults:
        """ Export Fluent Meshing mesh file. Fluent Meshing mesh files have .msh extension.


        Parameters
        ----------
        file_name : str
            Path to file on disk.
        export_fluent_mesh_params : ExportFluentMeshingMeshParams
            Parameters to export Fluent Meshing mesh file.

        Returns
        -------
        FileWriteResults
            Returns the FileWriteResults structure.


        Examples
        --------
        >>> results = file_io.export_fluent_meshing_mesh("/tmp/fluent_meshing.msh", ExportFluentMeshingMeshParams(model = model))

        """
        if not isinstance(file_name, str):
            raise TypeError("Invalid argument type passed for file_name, valid argument type is str.")
        if not isinstance(export_fluent_mesh_params, ExportFluentMeshingMeshParams):
            raise TypeError("Invalid argument type passed for export_fluent_mesh_params, valid argument type is ExportFluentMeshingMeshParams.")
        args = {"file_name" : file_name,
        "export_fluent_mesh_params" : export_fluent_mesh_params._jsonify()}
        command_name = "PrimeMesh::FileIO/ExportFluentMeshingMesh"
        self._model._print_logs_before_command("export_fluent_meshing_mesh", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("export_fluent_meshing_mesh", FileWriteResults(model = self._model, json_data = result))
        return FileWriteResults(model = self._model, json_data = result)

    def export_lsdyna_iga_keyword_file(self, file_name : str, export_params : ExportLSDynaIgaKeywordFileParams) -> FileWriteResults:
        """ Export IGA LS-DYNA Keyword file for solid, surface splines, or both.


        Parameters
        ----------
        file_name : str
            Name of the file.
        export_params : ExportLSDynaIgaKeywordFileParams
            Parameters for IGA LS-DYNA Keyword file export.

        Returns
        -------
        FileWriteResults
            Returns FileWriteResults.

        Notes
        -----
        This API is a Beta. API Behavior and implementation may change in future.

        Examples
        --------
        >>> results = file_io.export_lsdyna_iga_keyword_file(file_name, ExportLSDynaIgaKeywordFileParams(model = model))

        """
        if not isinstance(file_name, str):
            raise TypeError("Invalid argument type passed for file_name, valid argument type is str.")
        if not isinstance(export_params, ExportLSDynaIgaKeywordFileParams):
            raise TypeError("Invalid argument type passed for export_params, valid argument type is ExportLSDynaIgaKeywordFileParams.")
        args = {"file_name" : file_name,
        "export_params" : export_params._jsonify()}
        command_name = "PrimeMesh::FileIO/ExportLSDynaIgaKeywordFile"
        self._model._print_beta_api_warning("export_lsdyna_iga_keyword_file")
        self._model._print_logs_before_command("export_lsdyna_iga_keyword_file", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("export_lsdyna_iga_keyword_file", FileWriteResults(model = self._model, json_data = result))
        return FileWriteResults(model = self._model, json_data = result)

    def export_boundary_fitted_spline_kfile(self, file_name : str, export_params : ExportBoundaryFittedSplineParams) -> FileWriteResults:
        """ Export IGA LS-DYNA Keyword file for boundary fitted spline.


        Parameters
        ----------
        file_name : str
            Name of the file.
        export_params : ExportBoundaryFittedSplineParams
            Parameters for IGA LS-DYNA Keyword file export.

        Returns
        -------
        FileWriteResults
            Returns FileWriteResults.

        Examples
        --------
        >>> results = file_io.export_boundary_fitted_spline_k_file(file_name, ExportBoundaryFittedSplineParams(model = model))

        """
        if not isinstance(file_name, str):
            raise TypeError("Invalid argument type passed for file_name, valid argument type is str.")
        if not isinstance(export_params, ExportBoundaryFittedSplineParams):
            raise TypeError("Invalid argument type passed for export_params, valid argument type is ExportBoundaryFittedSplineParams.")
        args = {"file_name" : file_name,
        "export_params" : export_params._jsonify()}
        command_name = "PrimeMesh::FileIO/ExportBoundaryFittedSplineKFile"
        self._model._print_logs_before_command("export_boundary_fitted_spline_kfile", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("export_boundary_fitted_spline_kfile", FileWriteResults(model = self._model, json_data = result))
        return FileWriteResults(model = self._model, json_data = result)

    def import_cad(self, file_name : str, import_cad_params : ImportCadParams) -> ImportCadResults:
        """ Imports CAD file from disk.


        Parameters
        ----------
        file_name : str
            Path to file on disk.
        import_cad_params : ImportCadParams
            Parameters to control CAD import options.

        Returns
        -------
        ImportCadResults
            Returns the ImportCadResults.


        Notes
        -----
        This API does not support Unicode paths now.

        Examples
        --------
        >>> file_io = prime.FileIO(model = model)
        >>> params = prime.ImportCadParams(model = model)
        >>> results = file_io.import_cad(file_name="/tmp/my_cad.x_t", import_cad_params=params)

        """
        if not isinstance(file_name, str):
            raise TypeError("Invalid argument type passed for file_name, valid argument type is str.")
        if not isinstance(import_cad_params, ImportCadParams):
            raise TypeError("Invalid argument type passed for import_cad_params, valid argument type is ImportCadParams.")
        args = {"file_name" : file_name,
        "import_cad_params" : import_cad_params._jsonify()}
        command_name = "PrimeMesh::FileIO/ImportCAD"
        self._model._print_logs_before_command("import_cad", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("import_cad", ImportCadResults(model = self._model, json_data = result))
        return ImportCadResults(model = self._model, json_data = result)

    def export_stl(self, file_name : str, params : ExportSTLParams) -> FileWriteResults:
        """ Export STL file.


        Parameters
        ----------
        file_name : str
            Path to file on disk.
        params : ExportSTLParams
            Parameters for writing the file.

        Returns
        -------
        FileWriteResults
            Returns the FileWriteResults.


        Notes
        -----
        This API does not support Unicode paths now.

        Examples
        --------
        >>> fileio = prime.FileIO(model=model)
        >>> out_file_path = r"/tmp/output.stl"
        >>> part_ids = [part.id for part in model.parts]
        >>> export_stl_params=prime.ExportSTLParams(model=model,part_ids=part_ids)
        >>> results = fileio.export_stl(out_file_path,export_stl_params)

        """
        if not isinstance(file_name, str):
            raise TypeError("Invalid argument type passed for file_name, valid argument type is str.")
        if not isinstance(params, ExportSTLParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is ExportSTLParams.")
        args = {"file_name" : file_name,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::FileIO/ExportSTL"
        self._model._print_logs_before_command("export_stl", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("export_stl", FileWriteResults(model = self._model, json_data = result))
        return FileWriteResults(model = self._model, json_data = result)



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class ThinVolumeControl(CoreObject):
    """ThinVolumeControl allows you to generate prisms in the space between surfaces.

    ThinVolumeControl allows you to control generation of prisms in the thin space between surfaces. Controls include setting the source face scope, target face scope and thin volume mesh parameters.
    """

    def __init__(self, model: CommunicationManager, id: int, object_id: int, name: str):
        """ Initialize ThinVolumeControl """
        self._model = model
        self._comm = model._communicator
        self._id = id
        self._object_id = object_id
        self._name = name
        self._freeze()

    def set_thin_volume_mesh_params(self, thin_volume_mesh_params : ThinVolumeMeshParams):
        """ Sets thin volume mesh parameters for thin volume control.


        Parameters
        ----------
        thin_volume_mesh_params : ThinVolumeMeshParams
            Parameters to set thin volume control growth.

        Examples
        --------
        >>> results = thin_vol_ctrl.set_thin_volume_mesh_params(ThinVolumeMeshParams(model=model))

        """
        if not isinstance(thin_volume_mesh_params, ThinVolumeMeshParams):
            raise TypeError("Invalid argument type passed for thin_volume_mesh_params, valid argument type is ThinVolumeMeshParams.")
        args = {"thin_volume_mesh_params" : thin_volume_mesh_params._jsonify()}
        command_name = "PrimeMesh::ThinVolumeControl/SetThinVolumeMeshParams"
        self._model._print_logs_before_command("set_thin_volume_mesh_params", args)
        self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_thin_volume_mesh_params")

    def set_source_scope(self, entities : ScopeDefinition) -> SetScopeResults:
        """ Sets the source surface scope of thin volume control.


        Parameters
        ----------
        entities : ScopeDefinition
            Scope definition entities.

        Returns
        -------
        SetScopeResults
            Returns SetScopeResults.


        Examples
        --------
        >>> results = thin_vol_ctrl.set_source_scope(entities)

        """
        if not isinstance(entities, ScopeDefinition):
            raise TypeError("Invalid argument type passed for entities, valid argument type is ScopeDefinition.")
        args = {"entities" : entities._jsonify()}
        command_name = "PrimeMesh::ThinVolumeControl/SetSourceScope"
        self._model._print_logs_before_command("set_source_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_source_scope", SetScopeResults(model = self._model, json_data = result))
        return SetScopeResults(model = self._model, json_data = result)

    def set_target_scope(self, entities : ScopeDefinition) -> SetScopeResults:
        """ Sets the target surface scope of thin volume control.


        Parameters
        ----------
        entities : ScopeDefinition
            Scope definition entities.

        Returns
        -------
        SetScopeResults
            Returns SetScopeResults.


        Examples
        --------
        >>> results = thin_vol_ctrl.set_target_scope(entities)

        """
        if not isinstance(entities, ScopeDefinition):
            raise TypeError("Invalid argument type passed for entities, valid argument type is ScopeDefinition.")
        args = {"entities" : entities._jsonify()}
        command_name = "PrimeMesh::ThinVolumeControl/SetTargetScope"
        self._model._print_logs_before_command("set_target_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_target_scope", SetScopeResults(model = self._model, json_data = result))
        return SetScopeResults(model = self._model, json_data = result)

    @property
    def id(self):
        """ Get the id of ThinVolumeControl."""
        return self._id

    @property
    def name(self):
        """ Get the name of ThinVolumeControl."""
        return self._name



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class Surfer(CoreObject):
    """Generates surface mesh.

    Performs surface meshing using various surface meshing algorithms on topofaces or face zonelets.
    For example, constant size or volumetric size surface meshing.
    """

    def __init__(self, model: CommunicationManager, part_id: int):
        """ Initialize Surfer  """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::Surfer/Construct"
        args = {"ModelID" : model._object_id , "PartID" : part_id, "MaxID" : -1}
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for Surfer. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for Surfer. """
        command_name = "PrimeMesh::Surfer/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def remesh_face_zonelets(self, face_zonelets : Iterable[int], edge_zonelets : Iterable[int], params : SurferParams) -> SurferResults:
        """ Performs meshing on the given face zonelets with provided parameters.


        Parameters
        ----------
        face_zonelets : Iterable[int]
            Ids of face zonelets.
        edge_zonelets : Iterable[int]
            Ids of edge zonelets.
        params : SurferParams
            Surfer parameters.

        Returns
        -------
        SurferResults
            Returns the SurferResults.


        Examples
        --------
        >>> results = surfer.remesh_face_zonelets(face_zonelets, edge_zonelets, params)

        """
        if not isinstance(face_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for face_zonelets, valid argument type is Iterable[int].")
        if not isinstance(edge_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for edge_zonelets, valid argument type is Iterable[int].")
        if not isinstance(params, SurferParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is SurferParams.")
        args = {"face_zonelets" : face_zonelets,
        "edge_zonelets" : edge_zonelets,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Surfer/RemeshFaceZonelets"
        self._model._print_logs_before_command("remesh_face_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("remesh_face_zonelets", SurferResults(model = self._model, json_data = result))
        return SurferResults(model = self._model, json_data = result)

    def mesh_topo_faces(self, topo_faces : Iterable[int], params : SurferParams) -> SurferResults:
        """ Performs meshing on the given topofaces with provided parameters.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of topofaces.
        params : SurferParams
            Surfer Parameters.

        Returns
        -------
        SurferResults
            Returns the SurferResults.


        Examples
        --------
        >>> results = surfer.mesh_topo_faces(topo_faces, params)

        """
        if not isinstance(topo_faces, Iterable):
            raise TypeError("Invalid argument type passed for topo_faces, valid argument type is Iterable[int].")
        if not isinstance(params, SurferParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is SurferParams.")
        args = {"topo_faces" : topo_faces,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Surfer/MeshTopoFaces"
        self._model._print_logs_before_command("mesh_topo_faces", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("mesh_topo_faces", SurferResults(model = self._model, json_data = result))
        return SurferResults(model = self._model, json_data = result)

    def initialize_surfer_params_for_wrapper(self) -> SurferParams:
        """ Initializes surfer parameters to mesh surfaces generated by wrapper.


        Returns
        -------
        SurferParams
            Returns the SurferParams initialized for wrapper inputs.


        Examples
        --------
        >>> surfer_params = surfer.initialize_surfer_params_for_wrapper()

        """
        args = {}
        command_name = "PrimeMesh::Surfer/InitializeSurferParamsForWrapper"
        self._model._print_logs_before_command("initialize_surfer_params_for_wrapper", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("initialize_surfer_params_for_wrapper", SurferParams(model = self._model, json_data = result))
        return SurferParams(model = self._model, json_data = result)

    def remesh_face_zonelets_locally(self, face_zonelets : Iterable[int], register_id : int, local_surfer_params : LocalSurferParams) -> LocalSurferResults:
        """ Remesh the given face zonelets locally at the registered faces with provided parameters.


        Parameters
        ----------
        face_zonelets : Iterable[int]
            Ids of face zonelets.
        register_id : int
            Register id of the target faces.
        local_surfer_params : LocalSurferParams
            Local surfer Parameters.

        Returns
        -------
        LocalSurferResults
            Returns the LocalSurferResults.


        Examples
        --------
        >>> results = surfer.remesh_face_zonelets_locally(face_zonelets, register_id, local_surfer_params)

        """
        if not isinstance(face_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for face_zonelets, valid argument type is Iterable[int].")
        if not isinstance(register_id, int):
            raise TypeError("Invalid argument type passed for register_id, valid argument type is int.")
        if not isinstance(local_surfer_params, LocalSurferParams):
            raise TypeError("Invalid argument type passed for local_surfer_params, valid argument type is LocalSurferParams.")
        args = {"face_zonelets" : face_zonelets,
        "register_id" : register_id,
        "local_surfer_params" : local_surfer_params._jsonify()}
        command_name = "PrimeMesh::Surfer/RemeshFaceZoneletsLocally"
        self._model._print_logs_before_command("remesh_face_zonelets_locally", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("remesh_face_zonelets_locally", LocalSurferResults(model = self._model, json_data = result))
        return LocalSurferResults(model = self._model, json_data = result)



""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class SFPeriodicParams(CoreObject):
    """Parameters for setting periodic size fields.
    """
    _default_params = {}

    def __initialize(
            self,
            axis: Iterable[float],
            angle: float,
            center: Iterable[float]):
        self._axis = axis if isinstance(axis, np.ndarray) else np.array(axis, dtype=np.double) if axis is not None else None
        self._angle = angle
        self._center = center if isinstance(center, np.ndarray) else np.array(center, dtype=np.double) if center is not None else None

    def __init__(
            self,
            model: CommunicationManager=None,
            axis: Iterable[float] = None,
            angle: float = None,
            center: Iterable[float] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SFPeriodicParams.

        Parameters
        ----------
        model: Model
            Model to create a SFPeriodicParams object with default parameters.
        axis: Iterable[float], optional
            Axis vector coordinates.
        angle: float, optional
            Angle in degrees.
        center: Iterable[float], optional
            Center coordinates.
        json_data: dict, optional
            JSON dictionary to create a SFPeriodicParams object with provided parameters.

        Examples
        --------
        >>> s_fperiodic_params = prime.SFPeriodicParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["axis"] if "axis" in json_data else None,
                json_data["angle"] if "angle" in json_data else None,
                json_data["center"] if "center" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [axis, angle, center])
            if all_field_specified:
                self.__initialize(
                    axis,
                    angle,
                    center)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SFPeriodicParams")
                    json_data = param_json["SFPeriodicParams"] if "SFPeriodicParams" in param_json else {}
                    self.__initialize(
                        axis if axis is not None else ( SFPeriodicParams._default_params["axis"] if "axis" in SFPeriodicParams._default_params else (json_data["axis"] if "axis" in json_data else None)),
                        angle if angle is not None else ( SFPeriodicParams._default_params["angle"] if "angle" in SFPeriodicParams._default_params else (json_data["angle"] if "angle" in json_data else None)),
                        center if center is not None else ( SFPeriodicParams._default_params["center"] if "center" in SFPeriodicParams._default_params else (json_data["center"] if "center" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            axis: Iterable[float] = None,
            angle: float = None,
            center: Iterable[float] = None):
        """Set the default values of SFPeriodicParams.

        Parameters
        ----------
        axis: Iterable[float], optional
            Axis vector coordinates.
        angle: float, optional
            Angle in degrees.
        center: Iterable[float], optional
            Center coordinates.
        """
        args = locals()
        [SFPeriodicParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SFPeriodicParams.

        Examples
        --------
        >>> SFPeriodicParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SFPeriodicParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._axis is not None:
            json_data["axis"] = self._axis
        if self._angle is not None:
            json_data["angle"] = self._angle
        if self._center is not None:
            json_data["center"] = self._center
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "axis :  %s\nangle :  %s\ncenter :  %s" % (self._axis, self._angle, self._center)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def axis(self) -> Iterable[float]:
        """Axis vector coordinates.
        """
        return self._axis

    @axis.setter
    def axis(self, value: Iterable[float]):
        self._axis = value

    @property
    def angle(self) -> float:
        """Angle in degrees.
        """
        return self._angle

    @angle.setter
    def angle(self, value: float):
        self._angle = value

    @property
    def center(self) -> Iterable[float]:
        """Center coordinates.
        """
        return self._center

    @center.setter
    def center(self, value: Iterable[float]):
        self._center = value

class VolumetricSizeFieldComputeResults(CoreObject):
    """Results associated with the compute volumetric size field operation.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            size_field_id: int):
        self._error_code = ErrorCode(error_code)
        self._size_field_id = size_field_id

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            size_field_id: int = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the VolumetricSizeFieldComputeResults.

        Parameters
        ----------
        model: Model
            Model to create a VolumetricSizeFieldComputeResults object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with the compute volumetric size field operation.
        size_field_id: int, optional
            Id of the computed size field.
        json_data: dict, optional
            JSON dictionary to create a VolumetricSizeFieldComputeResults object with provided parameters.

        Examples
        --------
        >>> volumetric_size_field_compute_results = prime.VolumetricSizeFieldComputeResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                json_data["sizeFieldId"] if "sizeFieldId" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, size_field_id])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    size_field_id)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "VolumetricSizeFieldComputeResults")
                    json_data = param_json["VolumetricSizeFieldComputeResults"] if "VolumetricSizeFieldComputeResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( VolumetricSizeFieldComputeResults._default_params["error_code"] if "error_code" in VolumetricSizeFieldComputeResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        size_field_id if size_field_id is not None else ( VolumetricSizeFieldComputeResults._default_params["size_field_id"] if "size_field_id" in VolumetricSizeFieldComputeResults._default_params else (json_data["sizeFieldId"] if "sizeFieldId" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            size_field_id: int = None):
        """Set the default values of VolumetricSizeFieldComputeResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with the compute volumetric size field operation.
        size_field_id: int, optional
            Id of the computed size field.
        """
        args = locals()
        [VolumetricSizeFieldComputeResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of VolumetricSizeFieldComputeResults.

        Examples
        --------
        >>> VolumetricSizeFieldComputeResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in VolumetricSizeFieldComputeResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._size_field_id is not None:
            json_data["sizeFieldId"] = self._size_field_id
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\nsize_field_id :  %s" % (self._error_code, self._size_field_id)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the compute volumetric size field operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def size_field_id(self) -> int:
        """Id of the computed size field.
        """
        return self._size_field_id

    @size_field_id.setter
    def size_field_id(self, value: int):
        self._size_field_id = value

class VolumetricSizeFieldComputeParams(CoreObject):
    """Parameters associated with the compute volumetric size field operation.
    """
    _default_params = {}

    def __initialize(
            self,
            enable_multi_threading: bool,
            enable_periodicity: bool,
            periodic_params: SFPeriodicParams):
        self._enable_multi_threading = enable_multi_threading
        self._enable_periodicity = enable_periodicity
        self._periodic_params = periodic_params

    def __init__(
            self,
            model: CommunicationManager=None,
            enable_multi_threading: bool = None,
            enable_periodicity: bool = None,
            periodic_params: SFPeriodicParams = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the VolumetricSizeFieldComputeParams.

        Parameters
        ----------
        model: Model
            Model to create a VolumetricSizeFieldComputeParams object with default parameters.
        enable_multi_threading: bool, optional
            Option to compute volumetric size field in parallel using multithreads.
        enable_periodicity: bool, optional
            Option to enable periodic size field computations.
        periodic_params: SFPeriodicParams, optional
            Periodic parameters to compute the size field.
        json_data: dict, optional
            JSON dictionary to create a VolumetricSizeFieldComputeParams object with provided parameters.

        Examples
        --------
        >>> volumetric_size_field_compute_params = prime.VolumetricSizeFieldComputeParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["enableMultiThreading"] if "enableMultiThreading" in json_data else None,
                json_data["enablePeriodicity"] if "enablePeriodicity" in json_data else None,
                SFPeriodicParams(model = model, json_data = json_data["periodicParams"] if "periodicParams" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [enable_multi_threading, enable_periodicity, periodic_params])
            if all_field_specified:
                self.__initialize(
                    enable_multi_threading,
                    enable_periodicity,
                    periodic_params)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "VolumetricSizeFieldComputeParams")
                    json_data = param_json["VolumetricSizeFieldComputeParams"] if "VolumetricSizeFieldComputeParams" in param_json else {}
                    self.__initialize(
                        enable_multi_threading if enable_multi_threading is not None else ( VolumetricSizeFieldComputeParams._default_params["enable_multi_threading"] if "enable_multi_threading" in VolumetricSizeFieldComputeParams._default_params else (json_data["enableMultiThreading"] if "enableMultiThreading" in json_data else None)),
                        enable_periodicity if enable_periodicity is not None else ( VolumetricSizeFieldComputeParams._default_params["enable_periodicity"] if "enable_periodicity" in VolumetricSizeFieldComputeParams._default_params else (json_data["enablePeriodicity"] if "enablePeriodicity" in json_data else None)),
                        periodic_params if periodic_params is not None else ( VolumetricSizeFieldComputeParams._default_params["periodic_params"] if "periodic_params" in VolumetricSizeFieldComputeParams._default_params else SFPeriodicParams(model = model, json_data = (json_data["periodicParams"] if "periodicParams" in json_data else None))))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            enable_multi_threading: bool = None,
            enable_periodicity: bool = None,
            periodic_params: SFPeriodicParams = None):
        """Set the default values of VolumetricSizeFieldComputeParams.

        Parameters
        ----------
        enable_multi_threading: bool, optional
            Option to compute volumetric size field in parallel using multithreads.
        enable_periodicity: bool, optional
            Option to enable periodic size field computations.
        periodic_params: SFPeriodicParams, optional
            Periodic parameters to compute the size field.
        """
        args = locals()
        [VolumetricSizeFieldComputeParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of VolumetricSizeFieldComputeParams.

        Examples
        --------
        >>> VolumetricSizeFieldComputeParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in VolumetricSizeFieldComputeParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._enable_multi_threading is not None:
            json_data["enableMultiThreading"] = self._enable_multi_threading
        if self._enable_periodicity is not None:
            json_data["enablePeriodicity"] = self._enable_periodicity
        if self._periodic_params is not None:
            json_data["periodicParams"] = self._periodic_params._jsonify()
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "enable_multi_threading :  %s\nenable_periodicity :  %s\nperiodic_params :  %s" % (self._enable_multi_threading, self._enable_periodicity, '{ ' + str(self._periodic_params) + ' }')
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def enable_multi_threading(self) -> bool:
        """Option to compute volumetric size field in parallel using multithreads.
        """
        return self._enable_multi_threading

    @enable_multi_threading.setter
    def enable_multi_threading(self, value: bool):
        self._enable_multi_threading = value

    @property
    def enable_periodicity(self) -> bool:
        """Option to enable periodic size field computations.
        """
        return self._enable_periodicity

    @enable_periodicity.setter
    def enable_periodicity(self, value: bool):
        self._enable_periodicity = value

    @property
    def periodic_params(self) -> SFPeriodicParams:
        """Periodic parameters to compute the size field.
        """
        return self._periodic_params

    @periodic_params.setter
    def periodic_params(self, value: SFPeriodicParams):
        self._periodic_params = value



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from typing import List, Any, Union

class CoreObject(object):
    """

    """

    __isfrozen = False

    def __setattr__(self, key, value) :
        """ __setattr__(CoreObject self, key, value)"""
        if self.__isfrozen and not hasattr(self, key) :
           raise TypeError("%r is an invalid attribute" % key)
        object.__setattr__(self, key, value)

    def _freeze(self) :
        """ _freeze(CoreObject self)"""
        self.__isfrozen = True

    def _unfreeze(self) :
        """ _freeze(CoreObject self)"""
        self.__isfrozen = False



import os
from typing import Optional, Union

from .download_utilities import DownloadManager

FILE_NAMES = [
    "box.psf",
    "box.sf",
    "file.cas",
    "file.pdmat",
    "file.pmdat",
    "hex.cas",
    "hex.cdb",
    "hex.fmd",
    "hex.msh",
]


def download_test_examples(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download the example files necessary for unit testing.

    Parameters
    ----------
    destination : str, optional
        Path to download the test example files to. The default
        is ``None``, in which case the default path for app data
        is used.
    force : bool, optional
        Whether to always download the test example files. The default is
        ``False``, in which case if the test example files are cached, they
        are reused.

    Returns
    -------
    List[str]
        Local paths to the downloaded test example files.

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     examples = prime_examples.unit_test_examples.download_test_examples()


    """

    download_manager = DownloadManager()
    unit_test_paths = [
        download_manager.download_file(
            file, 'pyprimemesh', 'unit_test_examples', destination=destination, force=force
        )
        for file in FILE_NAMES
    ]
    return unit_test_paths



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class MultiZoneControl(CoreObject):
    """MultiZoneControl provides a way to gather all the information required for MultiZone meshing.

    The MultiZone meshing provides hex meshing capabilities.
    Different type of mesh can be generated using MultiZoneControl. For example, sweep mesh, map mesh and edge biased mesh.
    """

    def __init__(self, model: CommunicationManager, id: int, object_id: int, name: str):
        """ Initialize MultiZoneControl """
        self._model = model
        self._comm = model._communicator
        self._id = id
        self._object_id = object_id
        self._name = name
        self._freeze()

    def set_volume_scope(self, scope_info : ScopeDefinition):
        """ Sets the scope for volume in terms of topovolumes.The topovolumes given by the scope are imported by MultiZone to generate MultiZone mesh on it.


        Parameters
        ----------
        scope_info : ScopeDefinition
            Scoped topovolumes.

        Notes
        -----
        This API is a Beta. API Behavior and implementation may change in future.

        Examples
        --------
        >>> multizone_control.set_volume_scope(scope_info)

        """
        if not isinstance(scope_info, ScopeDefinition):
            raise TypeError("Invalid argument type passed for scope_info, valid argument type is ScopeDefinition.")
        args = {"scope_info" : scope_info._jsonify()}
        command_name = "PrimeMesh::MultiZoneControl/SetVolumeScope"
        self._model._print_beta_api_warning("set_volume_scope")
        self._model._print_logs_before_command("set_volume_scope", args)
        self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_volume_scope")

    def set_surface_scope(self, scope_info : ScopeDefinition):
        """ Sets the scope for surface in terms of topofaces.


        Parameters
        ----------
        scope_info : ScopeDefinition
            Scoped topofaces.

        Notes
        -----
        This API is a Beta. API Behavior and implementation may change in future.

        Examples
        --------
        >>> multizone_control.set_surface_scope(scope_info)

        """
        if not isinstance(scope_info, ScopeDefinition):
            raise TypeError("Invalid argument type passed for scope_info, valid argument type is ScopeDefinition.")
        args = {"scope_info" : scope_info._jsonify()}
        command_name = "PrimeMesh::MultiZoneControl/SetSurfaceScope"
        self._model._print_beta_api_warning("set_surface_scope")
        self._model._print_logs_before_command("set_surface_scope", args)
        self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_surface_scope")

    def set_map_mesh_params(self, scope_info : MultiZoneMapMeshParams):
        """ Sets the parameters for map meshing in terms of topofaces during MultiZone mesh.


        Parameters
        ----------
        scope_info : MultiZoneMapMeshParams
            Scoped topofaces to be map meshed.

        Notes
        -----
        This API is a Beta. API Behavior and implementation may change in future.

        Examples
        --------
        >>> multizone_control.set_map_mesh_params(scope_info)

        """
        if not isinstance(scope_info, MultiZoneMapMeshParams):
            raise TypeError("Invalid argument type passed for scope_info, valid argument type is MultiZoneMapMeshParams.")
        args = {"scope_info" : scope_info._jsonify()}
        command_name = "PrimeMesh::MultiZoneControl/SetMapMeshParams"
        self._model._print_beta_api_warning("set_map_mesh_params")
        self._model._print_logs_before_command("set_map_mesh_params", args)
        self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_map_mesh_params")

    def set_sweep_mesh_params(self, scope_info : MultiZoneSweepMeshParams):
        """ Sets the parameters for sweep meshing in terms of topofaces during MultiZone mesh.


        Parameters
        ----------
        scope_info : MultiZoneSweepMeshParams
            Information required for sweep meshing.

        Notes
        -----
        This API is a Beta. API Behavior and implementation may change in future.

        Examples
        --------
        >>> multizone_control.set_sweep_mesh_params(scope_info)

        """
        if not isinstance(scope_info, MultiZoneSweepMeshParams):
            raise TypeError("Invalid argument type passed for scope_info, valid argument type is MultiZoneSweepMeshParams.")
        args = {"scope_info" : scope_info._jsonify()}
        command_name = "PrimeMesh::MultiZoneControl/SetSweepMeshParams"
        self._model._print_beta_api_warning("set_sweep_mesh_params")
        self._model._print_logs_before_command("set_sweep_mesh_params", args)
        self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_sweep_mesh_params")

    def set_edge_biasing_params(self, scope_info : MultiZoneEdgeBiasingParams):
        """ Sets the parameters for edge biasing in terms of topoedges and topofaces during MultiZone mesh.


        Parameters
        ----------
        scope_info : MultiZoneEdgeBiasingParams
            Information for edge biasing in MultiZone meshing.

        Notes
        -----
        This API is a Beta. API Behavior and implementation may change in future.

        Examples
        --------
        >>> multizone_control.set_edge_biasing_params(scope_info)

        """
        if not isinstance(scope_info, MultiZoneEdgeBiasingParams):
            raise TypeError("Invalid argument type passed for scope_info, valid argument type is MultiZoneEdgeBiasingParams.")
        args = {"scope_info" : scope_info._jsonify()}
        command_name = "PrimeMesh::MultiZoneControl/SetEdgeBiasingParams"
        self._model._print_beta_api_warning("set_edge_biasing_params")
        self._model._print_logs_before_command("set_edge_biasing_params", args)
        self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_edge_biasing_params")

    def set_multi_zone_params(self, params : MultiZoneParams):
        """ Sets the MultiZone parameters.


        Parameters
        ----------
        params : MultiZoneParams
            Parameters required for MultiZone mesh.

        Notes
        -----
        This API is a Beta. API Behavior and implementation may change in future.

        Examples
        --------
        >>> multizone_control.set_multi_zone_params(params)

        """
        if not isinstance(params, MultiZoneParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is MultiZoneParams.")
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::MultiZoneControl/SetMultiZoneParams"
        self._model._print_beta_api_warning("set_multi_zone_params")
        self._model._print_logs_before_command("set_multi_zone_params", args)
        self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_multi_zone_params")

    @property
    def id(self):
        """ Get the id of MultiZoneControl."""
        return self._id

    @property
    def name(self):
        """ Get the name of MultiZoneControl."""
        return self._name



"""Module for Lucid Scope for operation on surfaces functionality."""
from typing import Iterable

from ansys.meshing.prime.autogen.controlstructs import (
    ScopeDefinition,
    ScopeEntity,
    ScopeEvaluationType,
)
from ansys.meshing.prime.autogen.partstructs import NamePatternParams
from ansys.meshing.prime.core.model import Model

from .utils import check_name_pattern


class _LucidScope:
    def __init__(
        self,
        part_expression: str,
        entity_expression: str,
        scope_evaluation_type: ScopeEvaluationType,
        scope_entity_type: ScopeEntity,
    ):
        self._part_expression = part_expression
        self._entity_expression = entity_expression
        self._evaluation_type = scope_evaluation_type
        self._entity_type = scope_entity_type

    def __str__(self) -> str:
        return "#".join(
            [
                self._part_expression,
                self._entity_expression,
                str(int(self._evaluation_type)),
                str(int(self._entity_type)),
            ]
        )

    def get_scope_definition(self, model: Model) -> ScopeDefinition:
        """Get the scope definition of the scope.

        Parameters
        ----------
        model : Model
            PyPrimeMesh model.

        Returns
        -------
        ScopeDefinition
            Returns the scope definition.

        """
        label_exp: str = None
        zone_exp: str = None
        if self._evaluation_type == ScopeEvaluationType.LABELS:
            label_exp = self._entity_expression
        else:
            zone_exp = self._entity_expression

        sd = ScopeDefinition(
            model=model,
            entity_type=self._entity_type,
            evaluation_type=self._evaluation_type,
            part_expression=self._part_expression,
            label_expression=label_exp,
            zone_expression=zone_exp,
        )
        return sd


class SurfaceScope(_LucidScope):
    """SurfaceScope is one of the classes in Lucid API.

    This class is meant for beginners to meshing. This class is used to define
    a scope for operation on surfaces.

    """

    def __init__(
        self,
        part_expression: str = "*",
        entity_expression: str = "*",
        scope_evaluation_type: ScopeEvaluationType = ScopeEvaluationType.LABELS,
    ):
        """Initialize SurfaceScope.

        Initialize SurfaceScope with the given part expression,
        entity expression and scope evaluation type.

        Parameters
        ----------
        part_expression : str
            Part expression to scope parts while evaluating scope.
        entity_expression : str
            Label or zone expression to scope entities while evaluating scope.
        scope_evaluation_type : prime.ScopeEvaluationType
            Evaluation type to scope entities. The default is set to labels.

        """
        _LucidScope.__init__(
            self,
            part_expression,
            entity_expression,
            scope_evaluation_type,
            ScopeEntity.FACEZONELETS,
        )

    def get_parts(self, model: Model) -> Iterable[int]:
        """Get the list of part ids in the scope.

        Parameters
        ----------
        model : Model
            PyPrimeMesh model.

        Returns
        -------
        Iterable[int]
            Returns the list of part ids.

        Examples
        --------
            >>> from ansys.meshing.prime import Model, SurfaceScope
            >>> model = client.model
            >>> su = SurfaceScope("*", "*", prime.ScopeEvaluationType.LABELS)
            >>> part_ids = su.get_parts()

        """
        sel_parts: Iterable[int] = []
        for part in model.parts:
            if check_name_pattern(self._part_expression, part.name):
                sel_parts.append(part.id)
        return sel_parts

    def get_face_zonelets(self, model: Model, part_id: int) -> Iterable[int]:
        """Get the list of face zonelets for the given part in the scope.

        Parameters
        ----------
        model : Model
            PyPrimeMesh model.
        part_id : int
            Id of the part.

        Returns
        -------
        Iterable[int]
            Returns the list of zonelets.

        Examples
        --------
            >>> from ansys.meshing.prime import Model, SurfaceScope
            >>> model = client.model
            >>> su = SurfaceScope("*", "*", prime.ScopeEvaluationType.LABELS)
            >>> face_zonelets = su.get_face_zonelets(model, 2)

        """
        face_zonelets: Iterable[int] = []
        part = model.get_part(part_id)
        if part and check_name_pattern(self._part_expression, part.name):
            if self._evaluation_type == ScopeEvaluationType.LABELS:
                face_zonelets = part.get_face_zonelets_of_label_name_pattern(
                    self._entity_expression, NamePatternParams(model)
                )
            else:
                face_zonelets = part.get_face_zonelets_of_zone_name_pattern(
                    self._entity_expression, NamePatternParams(model)
                )
        return face_zonelets

    def get_topo_faces(self, model: Model, part_id: int) -> Iterable[int]:
        """Get the list of topofaces for the given part in the scope.

        Parameters
        ----------
        model : Model
            PyPrimeMesh model.
        part_id : int
            Id of the part.

        Returns
        -------
        Iterable[int]
            Returns the list of zonelets.

        Examples
        --------
            >>> from ansys.meshing.prime import Model, SurfaceScope
            >>> model = client.model
            >>> su = SurfaceScope("*", "*", prime.ScopeEvaluationType.LABELS)
            >>> topo_faces = su.get_topo_faces(model, 2)

        """
        topo_faces: Iterable[int] = []
        part = model.get_part(part_id)
        if part and check_name_pattern(self._part_expression, part.name):
            if self._evaluation_type == ScopeEvaluationType.LABELS:
                topo_faces = part.get_topo_faces_of_label_name_pattern(
                    self._entity_expression, NamePatternParams(model)
                )
            else:
                topo_faces = part.get_topo_faces_of_zone_name_pattern(
                    self._entity_expression, NamePatternParams(model)
                )
        return topo_faces


class VolumeScope(_LucidScope):
    """VolumeScope is one of the classes in Lucid API.

    This class is meant for beginners to meshing. This class is used to define
    a scope for operation on volumes.

    """

    def __init__(
        self,
        part_expression: str = "*",
        entity_expression: str = "*",
        scope_evaluation_type: ScopeEvaluationType = ScopeEvaluationType.ZONES,
    ):
        """Initialize VolumeScope.

        Initialize VolumeScope with the given part expression, entity expression and scope
        evaluation type.

        Parameters
        ----------
        part_expression : str
            Part expression to scope parts while evaluating scope.
        entity_expression : str
            Label or zone expression to scope entities while evaluating scope.
        scope_evaluation_type : ScopeEvaluationType
            Evaluation type to scope entities. The default is set to zones.

        """
        _LucidScope.__init__(
            self,
            part_expression,
            entity_expression,
            scope_evaluation_type,
            ScopeEntity.VOLUME,
        )
